@@ -148,6 +148,7 @@ rocsparse_status rocsparse_routine::dispatch_precision(const char       precisio
     case rocsparse_datatype_i32_r:
     case rocsparse_datatype_u32_r:
     case rocsparse_datatype_f16_r:
+    case rocsparse_datatype_bf16_r:
         return rocsparse_status_invalid_value;
     }
     return rocsparse_status_invalid_value;
@@ -501,7 +502,7 @@ rocsparse_status rocsparse_routine::dispatch_call(const Arguments& arg)
     switch(FNAME)
     {
         DEFINE_CASE_T(axpyi);
-        DEFINE_CASE_IT_X(bellmm, testing_spmm_bell);
+        DEFINE_CASE_IABCT_X(bellmm, testing_spmm_bell);
         DEFINE_CASE_IT_X(bellmm_batched, testing_spmm_batched_bell);
         DEFINE_CASE_T(bsrgeam);
         DEFINE_CASE_T(bsric0);
@@ -539,6 +540,7 @@ rocsparse_status rocsparse_routine::dispatch_call(const Arguments& arg)
         DEFINE_CASE_IJT_X(csrgemm_reuse, testing_spgemm_reuse_csr);
         DEFINE_CASE_IJAXYT_X(bsrmv, testing_spmv_bsr);
         DEFINE_CASE_IJAXYT_X(csrmv, testing_spmv_csr);
+        DEFINE_CASE_IJAXYT_X(csrmv_analysis, testing_spmv_csr_analysis);
         DEFINE_CASE_T(csrmv_managed);
         DEFINE_CASE_IJABCT_X(bsrmm, testing_spmm_bsr);
         DEFINE_CASE_IJAXYT_X(cscmv, testing_spmv_csc);
