@@ -97,6 +97,17 @@ void unit_check_general(
     ROCSPARSE_UNIT_CHECK(M, N, A, LDA, B, LDB, ASSERT_FLOAT_EQ);
 }
 
+template <>
+void unit_check_general(int64_t                   M,
+                        int64_t                   N,
+                        const rocsparse_bfloat16* A,
+                        int64_t                   LDA,
+                        const rocsparse_bfloat16* B,
+                        int64_t                   LDB)
+{
+    ROCSPARSE_UNIT_CHECK(M, N, A, LDA, B, LDB, ASSERT_FLOAT_EQ);
+}
+
 template <>
 void unit_check_general(
     int64_t M, int64_t N, const float* A, int64_t LDA, const float* B, int64_t LDB)
@@ -575,6 +586,12 @@ void unit_check_garray(rocsparse_datatype val_type,
         unit_check_segments<uint8_t>(size, (const uint8_t*)source, (const uint8_t*)target);
         break;
     }
+    case rocsparse_datatype_bf16_r:
+    {
+        unit_check_segments<rocsparse_bfloat16>(
+            size, (const rocsparse_bfloat16*)source, (const rocsparse_bfloat16*)t);
+        break;
+    }
     }
     CHECK_HIP_ERROR(rocsparse_hipHostFree(s));
     CHECK_HIP_ERROR(rocsparse_hipHostFree(t));
