@@ -56,7 +56,8 @@ DEF(rocsparse_datatype,
     rocsparse_datatype_i8_r,
     rocsparse_datatype_u8_r,
     rocsparse_datatype_i32_r,
-    rocsparse_datatype_u32_r);
+    rocsparse_datatype_u32_r,
+    rocsparse_datatype_bf16_r);
 
 DEF(rocsparse_index_base, rocsparse_index_base_zero, rocsparse_index_base_one);
 
@@ -118,7 +119,8 @@ DEF(rocsparse_spmv_alg,
     rocsparse_spmv_alg_csr_rowsplit,
     rocsparse_spmv_alg_ell,
     rocsparse_spmv_alg_coo_atomic,
-    rocsparse_spmv_alg_csr_lrb);
+    rocsparse_spmv_alg_csr_lrb,
+    rocsparse_spmv_alg_csr_nnzsplit);
 
 DEF(rocsparse_spsv_alg, rocsparse_spsv_alg_default);
 
@@ -197,6 +199,7 @@ bool rocsparse_datatype_from_name(rocsparse_datatype value, const char* name)
             CASE(rocsparse_datatype_u8_r);
             CASE(rocsparse_datatype_i32_r);
             CASE(rocsparse_datatype_u32_r);
+            CASE(rocsparse_datatype_bf16_r);
         }
     }
     return false;
@@ -422,6 +425,7 @@ bool rocsparse_spmv_alg_from_name(rocsparse_spmv_alg value, const char* name)
             CASE(rocsparse_spmv_alg_ell);
             CASE(rocsparse_spmv_alg_coo_atomic);
             CASE(rocsparse_spmv_alg_csr_lrb);
+            CASE(rocsparse_spmv_alg_csr_nnzsplit);
         }
     }
     return false;
