@@ -32,7 +32,7 @@
 template <typename T, template <typename...> class VECTOR>
 static void apply_toint(VECTOR<T>& data)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     const size_t size = data.size();
     for(size_t i = 0; i < size; ++i)
@@ -44,7 +44,7 @@ static void apply_toint(VECTOR<T>& data)
 template <template <typename...> class VECTOR>
 static void apply_toint(VECTOR<_Float16>& data)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     const size_t size = data.size();
     for(size_t i = 0; i < size; ++i)
@@ -56,7 +56,7 @@ static void apply_toint(VECTOR<_Float16>& data)
 template <template <typename...> class VECTOR>
 static void apply_toint(VECTOR<rocsparse_float_complex>& data)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     const size_t size = data.size();
     for(size_t i = 0; i < size; ++i)
@@ -70,7 +70,7 @@ static void apply_toint(VECTOR<rocsparse_float_complex>& data)
 template <template <typename...> class VECTOR>
 static void apply_toint(VECTOR<rocsparse_double_complex>& data)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     const size_t size = data.size();
     for(size_t i = 0; i < size; ++i)
@@ -169,7 +169,7 @@ struct spec
                                       rocsparse_fill_mode    uplo,
                                       rocsparse_storage_mode storage)
     {
-        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
         factory.init_csr(
             bsr_row_ptr, bsr_col_ind, bsr_val, Mb, Nb, nnzb, base, matrix_type, uplo, storage);
@@ -204,7 +204,7 @@ struct spec<T, rocsparse_int, rocsparse_int>
         rocsparse_fill_mode                                                          uplo,
         rocsparse_storage_mode                                                       storage)
     {
-        ROCSPARSE_CLIENTS_ROUTINE_TRACE
+        ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
         //
         // Initialize in case init_csr requires it as input.
@@ -279,7 +279,7 @@ void rocsparse_matrix_factory_file<MATRIX_INIT, T, I, J>::init_gebsr(
     rocsparse_fill_mode    uplo,
     rocsparse_storage_mode storage)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     switch(MATRIX_INIT)
     {
@@ -393,7 +393,7 @@ void rocsparse_matrix_factory_file<MATRIX_INIT, T, I, J>::init_csr(
     rocsparse_fill_mode    uplo,
     rocsparse_storage_mode storage)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     std::vector<I> row_ptr;
     std::vector<J> col_ind;
@@ -495,7 +495,7 @@ void rocsparse_matrix_factory_file<MATRIX_INIT, T, I, J>::init_coo(
     rocsparse_fill_mode    uplo,
     rocsparse_storage_mode storage)
 {
-    ROCSPARSE_CLIENTS_ROUTINE_TRACE
+    ROCSPARSE_CLIENTS_ROUTINE_TRACE;
 
     std::vector<I> row_ind;
     std::vector<I> col_ind;
@@ -605,6 +605,19 @@ template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx,
                                               int64_t,
                                               int64_t>;
 
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx,
+                                              rocsparse_bfloat16,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int64_t>;
+
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx, float, int32_t, int32_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx, float, int64_t, int32_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_mtx, float, int64_t, int64_t>;
@@ -663,6 +676,18 @@ template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocalution,
                                               _Float16,
                                               int64_t,
                                               int64_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocalution,
+                                              rocsparse_bfloat16,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocalution,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocalution,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int64_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocalution,
                                               float,
                                               int32_t,
@@ -739,6 +764,18 @@ template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocsparseio,
                                               _Float16,
                                               int64_t,
                                               int64_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocsparseio,
+                                              rocsparse_bfloat16,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocsparseio,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocsparseio,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int64_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_rocsparseio,
                                               float,
                                               int32_t,
@@ -808,6 +845,19 @@ template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
                                               int64_t,
                                               int64_t>;
 
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_bfloat16,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int64_t>;
+
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, float, int32_t, int32_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, float, int64_t, int32_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_smtx, float, int64_t, int64_t>;
@@ -866,6 +916,18 @@ template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
                                               _Float16,
                                               int64_t,
                                               int64_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_bfloat16,
+                                              int32_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int32_t>;
+template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx,
+                                              rocsparse_bfloat16,
+                                              int64_t,
+                                              int64_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx, float, int32_t, int32_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx, float, int64_t, int32_t>;
 template struct rocsparse_matrix_factory_file<rocsparse_matrix_file_bsmtx, float, int64_t, int64_t>;
