@@ -236,15 +236,23 @@ constexpr double gebsrmm_gbyte_count(rocsparse_int Mb,
     return (reads + writes) / 1e9;
 }
 
-template <typename T, typename I, typename J>
-constexpr double csrmm_gbyte_count(J M, I nnz_A, I nnz_B, I nnz_C, bool beta = false)
+template <typename A, typename B, typename C, typename I>
+constexpr double bellmm_gbyte_count(I Mb, I width, I block_dim, I nnz_B, I nnz_C, bool beta = false)
 {
-    return ((M + 1) * sizeof(I) + nnz_A * sizeof(J)
-            + (nnz_A + nnz_B + nnz_C + (beta ? nnz_C : 0)) * sizeof(T))
+    return (sizeof(A) * Mb * width * block_dim * block_dim + sizeof(I) * Mb * width
+            + sizeof(B) * nnz_B + sizeof(C) * (nnz_C + (beta ? nnz_C : 0)))
            / 1e9;
 }
 
-template <typename T, typename I, typename J>
+template <typename A, typename B, typename C, typename I, typename J>
+constexpr double csrmm_gbyte_count(J M, I nnz_A, I nnz_B, I nnz_C, bool beta = false)
+{
+    return ((M + 1) * sizeof(I) + nnz_A * sizeof(J) + nnz_A * sizeof(A) + nnz_B * sizeof(B)
+            + (nnz_C + (beta ? nnz_C : 0)) * sizeof(C))
+           / 1e9;
+}
+
+template <typename A, typename B, typename C, typename I, typename J>
 constexpr double csrmm_batched_gbyte_count(J    M,
                                            I    nnz_A,
                                            I    nnz_B,
@@ -255,27 +263,27 @@ constexpr double csrmm_batched_gbyte_count(J    M,
                                            bool beta = false)
 {
     // read A matrix
-    size_t readA = batch_count_A * ((M + 1) * sizeof(I) + nnz_A * sizeof(J) + nnz_A * sizeof(T));
+    size_t readA = batch_count_A * ((M + 1) * sizeof(I) + nnz_A * sizeof(J) + nnz_A * sizeof(A));
 
     // read B matrix
-    size_t readB = batch_count_B * nnz_B * sizeof(T);
+    size_t readB = batch_count_B * nnz_B * sizeof(B);
 
     // read C matrix
-    size_t readC = batch_count_C * (beta ? nnz_C : 0) * sizeof(T);
+    size_t readC = batch_count_C * (beta ? nnz_C : 0) * sizeof(C);
 
     // write C matrix
-    size_t writeC = batch_count_C * nnz_C * sizeof(T);
+    size_t writeC = batch_count_C * nnz_C * sizeof(C);
 
     return (readA + readB + readC + writeC) / 1e9;
 }
 
-template <typename T, typename I, typename J>
+template <typename A, typename B, typename C, typename I, typename J>
 constexpr double cscmm_gbyte_count(J N, I nnz_A, I nnz_B, I nnz_C, bool beta = false)
 {
-    return csrmm_gbyte_count<T>(N, nnz_A, nnz_B, nnz_C, beta);
+    return csrmm_gbyte_count<A, B, C>(N, nnz_A, nnz_B, nnz_C, beta);
 }
 
-template <typename T, typename I, typename J>
+template <typename A, typename B, typename C, typename I, typename J>
 constexpr double cscmm_batched_gbyte_count(J    N,
                                            I    nnz_A,
                                            I    nnz_B,
@@ -286,28 +294,29 @@ constexpr double cscmm_batched_gbyte_count(J    N,
                                            bool beta = false)
 {
     // read A matrix
-    size_t readA = batch_count_A * ((N + 1) * sizeof(I) + nnz_A * sizeof(J) + nnz_A * sizeof(T));
+    size_t readA = batch_count_A * ((N + 1) * sizeof(I) + nnz_A * sizeof(J) + nnz_A * sizeof(A));
 
     // read B matrix
-    size_t readB = batch_count_B * nnz_B * sizeof(T);
+    size_t readB = batch_count_B * nnz_B * sizeof(B);
 
     // read C matrix
-    size_t readC = batch_count_C * (beta ? nnz_C : 0) * sizeof(T);
+    size_t readC = batch_count_C * (beta ? nnz_C : 0) * sizeof(C);
 
     // write C matrix
-    size_t writeC = batch_count_C * nnz_C * sizeof(T);
+    size_t writeC = batch_count_C * nnz_C * sizeof(C);
 
     return (readA + readB + readC + writeC) / 1e9;
 }
 
-template <typename T, typename I>
+template <typename A, typename B, typename C, typename I>
 constexpr double coomm_gbyte_count(int64_t nnz_A, int64_t nnz_B, int64_t nnz_C, bool beta = false)
 {
-    return (2.0 * nnz_A * sizeof(I) + (nnz_A + nnz_B + nnz_C + (beta ? nnz_C : 0)) * sizeof(T))
+    return (2.0 * nnz_A * sizeof(I) + nnz_A * sizeof(A) + nnz_B * sizeof(B)
+            + (nnz_C + (beta ? nnz_C : 0)) * sizeof(C))
            / 1e9;
 }
 
-template <typename T, typename I>
+template <typename A, typename B, typename C, typename I>
 constexpr double coomm_batched_gbyte_count(I       M,
                                            int64_t nnz_A,
                                            int64_t nnz_B,
@@ -318,16 +327,16 @@ constexpr double coomm_batched_gbyte_count(I       M,
                                            bool    beta = false)
 {
     // read A matrix
-    size_t readA = batch_count_A * (nnz_A * sizeof(I) + nnz_A * sizeof(I) + nnz_A * sizeof(T));
+    size_t readA = batch_count_A * (nnz_A * sizeof(I) + nnz_A * sizeof(I) + nnz_A * sizeof(A));
 
     // read B matrix
-    size_t readB = batch_count_B * nnz_B * sizeof(T);
+    size_t readB = batch_count_B * nnz_B * sizeof(B);
 
     // read C matrix
-    size_t readC = batch_count_C * (beta ? nnz_C : 0) * sizeof(T);
+    size_t readC = batch_count_C * (beta ? nnz_C : 0) * sizeof(C);
 
     // write C matrix
-    size_t writeC = batch_count_C * nnz_C * sizeof(T);
+    size_t writeC = batch_count_C * nnz_C * sizeof(C);
 
     return (readA + readB + readC + writeC) / 1e9;
 }
