@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -35,7 +35,7 @@
 #include <omp.h>
 #endif
 
-template <typename T, typename I, typename J>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 struct rocsparse_host
 {
 
@@ -48,14 +48,14 @@ struct rocsparse_host
                         J                    K,
                         I                    nnz,
                         const T*             alpha,
-                        const T*             A,
+                        const A*             dense_A,
                         int64_t              lda,
-                        const T*             B,
+                        const B*             dense_B,
                         int64_t              ldb,
                         const T*             beta,
                         const I*             csr_row_ptr_C,
                         const J*             csr_col_ind_C,
-                        T*                   csr_val_C,
+                        C*                   csr_val_C,
                         rocsparse_index_base base_C);
 
     static void cscddmm(rocsparse_operation  trans_A,
@@ -67,14 +67,14 @@ struct rocsparse_host
                         J                    K,
                         I                    nnz,
                         const T*             alpha,
-                        const T*             A,
+                        const A*             dense_A,
                         int64_t              lda,
-                        const T*             B,
+                        const B*             dense_B,
                         int64_t              ldb,
                         const T*             beta,
                         const I*             csr_row_ptr_C,
                         const J*             csr_col_ind_C,
-                        T*                   csr_val_C,
+                        C*                   csr_val_C,
                         rocsparse_index_base base_C);
 
     static void cooddmm(rocsparse_operation  trans_A,
@@ -86,14 +86,14 @@ struct rocsparse_host
                         J                    K,
                         I                    nnz,
                         const T*             alpha,
-                        const T*             A,
+                        const A*             dense_A,
                         int64_t              lda,
-                        const T*             B,
+                        const B*             dense_B,
                         int64_t              ldb,
                         const T*             beta,
                         const I*             coo_row_ind_C,
                         const I*             coo_col_ind_C,
-                        T*                   coo_val_C,
+                        C*                   coo_val_C,
                         rocsparse_index_base base_C);
 
     static void cooaosddmm(rocsparse_operation  trans_A,
@@ -105,14 +105,14 @@ struct rocsparse_host
                            J                    K,
                            I                    nnz,
                            const T*             alpha,
-                           const T*             A,
+                           const A*             dense_A,
                            int64_t              lda,
-                           const T*             B,
+                           const B*             dense_B,
                            int64_t              ldb,
                            const T*             beta,
                            const I*             coo_row_ind_C,
                            const I*             coo_col_ind_C,
-                           T*                   coo_val_C,
+                           C*                   coo_val_C,
                            rocsparse_index_base base_C);
 
     static void ellddmm(rocsparse_operation  trans_A,
@@ -124,14 +124,14 @@ struct rocsparse_host
                         J                    K,
                         I                    nnz,
                         const T*             alpha,
-                        const T*             A,
+                        const A*             dense_A,
                         int64_t              lda,
-                        const T*             B,
+                        const B*             dense_B,
                         int64_t              ldb,
                         const T*             beta,
                         const J              ell_width,
                         const I*             ell_ind_C,
-                        T*                   ell_val_C,
+                        C*                   ell_val_C,
                         rocsparse_index_base base_C);
 };
 
@@ -146,14 +146,14 @@ struct rocsparse_host
  *    level 1 SPARSE
  * ===========================================================================
  */
-template <typename I, typename T>
+template <typename T, typename I, typename X, typename Y>
 void host_axpby(I                    size,
                 I                    nnz,
                 T                    alpha,
-                const T*             x_val,
+                const X*             x_val,
                 const I*             x_ind,
                 T                    beta,
-                T*                   y,
+                Y*                   y,
                 rocsparse_index_base base);
 
 template <typename I, typename X, typename Y, typename T>
@@ -407,26 +407,28 @@ void host_gemvi(I                    M,
  *    level 3 SPARSE
  * ===========================================================================
  */
-template <typename T>
-void host_bsrmm(rocsparse_handle          handle,
-                rocsparse_direction       dir,
-                rocsparse_operation       trans_A,
-                rocsparse_operation       trans_B,
-                rocsparse_int             mb,
-                rocsparse_int             n,
-                rocsparse_int             kb,
-                rocsparse_int             nnzb,
-                const T*                  alpha,
-                const rocsparse_mat_descr descr,
-                const T*                  bsr_val,
-                const rocsparse_int*      bsr_row_ptr,
-                const rocsparse_int*      bsr_col_ind,
-                rocsparse_int             block_dim,
-                const T*                  B,
-                int64_t                   ldb,
-                const T*                  beta,
-                T*                        C,
-                int64_t                   ldc);
+template <typename T, typename I, typename J, typename A, typename B, typename C>
+void host_bsrmm(rocsparse_handle     handle,
+                rocsparse_direction  dir,
+                rocsparse_operation  trans_A,
+                rocsparse_operation  trans_B,
+                J                    mb,
+                J                    n,
+                J                    kb,
+                I                    nnzb,
+                T                    alpha,
+                const A*             bsr_val,
+                const I*             bsr_row_ptr,
+                const J*             bsr_col_ind,
+                J                    block_dim,
+                const B*             dense_B,
+                int64_t              ldb,
+                rocsparse_order      order_B,
+                T                    beta,
+                C*                   dense_C,
+                int64_t              ldc,
+                rocsparse_order      order_C,
+                rocsparse_index_base base);
 
 template <typename T>
 void host_gebsrmm(rocsparse_handle          handle,
@@ -450,7 +452,7 @@ void host_gebsrmm(rocsparse_handle          handle,
                   T*                        C,
                   int64_t                   ldc);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_csrmm(J                    M,
                 J                    N,
                 J                    K,
@@ -459,18 +461,18 @@ void host_csrmm(J                    M,
                 T                    alpha,
                 const I*             csr_row_ptr_A,
                 const J*             csr_col_ind_A,
-                const T*             csr_val_A,
-                const T*             B,
+                const A*             csr_val_A,
+                const B*             dense_B,
                 int64_t              ldb,
                 rocsparse_order      order_B,
                 T                    beta,
-                T*                   C,
+                C*                   dense_C,
                 int64_t              ldc,
                 rocsparse_order      order_C,
                 rocsparse_index_base base,
                 bool                 force_conj_A);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_csrmm_batched(J                    M,
                         J                    N,
                         J                    K,
@@ -482,14 +484,14 @@ void host_csrmm_batched(J                    M,
                         T                    alpha,
                         const I*             csr_row_ptr_A,
                         const J*             csr_col_ind_A,
-                        const T*             csr_val_A,
-                        const T*             B,
+                        const A*             csr_val_A,
+                        const B*             dense_B,
                         int64_t              ldb,
                         J                    batch_count_B,
                         int64_t              batch_stride_B,
                         rocsparse_order      order_B,
                         T                    beta,
-                        T*                   C,
+                        C*                   dense_C,
                         int64_t              ldc,
                         J                    batch_count_C,
                         int64_t              batch_stride_C,
@@ -497,7 +499,7 @@ void host_csrmm_batched(J                    M,
                         rocsparse_index_base base,
                         bool                 force_conj_A);
 
-template <typename T, typename I>
+template <typename T, typename I, typename A, typename B, typename C>
 void host_coomm(I                    M,
                 I                    N,
                 I                    K,
@@ -507,17 +509,17 @@ void host_coomm(I                    M,
                 T                    alpha,
                 const I*             coo_row_ind_A,
                 const I*             coo_col_ind_A,
-                const T*             coo_val_A,
-                const T*             B,
+                const A*             coo_val_A,
+                const B*             dense_B,
                 int64_t              ldb,
                 rocsparse_order      order_B,
                 T                    beta,
-                T*                   C,
+                C*                   dense_C,
                 int64_t              ldc,
                 rocsparse_order      order_C,
                 rocsparse_index_base base);
 
-template <typename T, typename I>
+template <typename T, typename I, typename A, typename B, typename C>
 void host_coomm_batched(I                    M,
                         I                    N,
                         I                    K,
@@ -529,21 +531,21 @@ void host_coomm_batched(I                    M,
                         T                    alpha,
                         const I*             coo_row_ind_A,
                         const I*             coo_col_ind_A,
-                        const T*             coo_val_A,
-                        const T*             B,
+                        const A*             coo_val_A,
+                        const B*             dense_B,
                         int64_t              ldb,
                         I                    batch_count_B,
                         int64_t              batch_stride_B,
                         rocsparse_order      order_B,
                         T                    beta,
-                        T*                   C,
+                        C*                   dense_C,
                         int64_t              ldc,
                         I                    batch_count_C,
                         int64_t              batch_stride_C,
                         rocsparse_order      order_C,
                         rocsparse_index_base base);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_cscmm(J                   M,
                 J                   N,
                 J                   K,
@@ -552,17 +554,17 @@ void host_cscmm(J                   M,
                 T                   alpha,
                 const I* __restrict csc_col_ptr_A,
                 const J* __restrict csc_row_ind_A,
-                const T* __restrict csc_val_A,
-                const T* __restrict B,
+                const A* __restrict csc_val_A,
+                const B* __restrict dense_B,
                 int64_t         ldb,
                 rocsparse_order order_B,
                 T               beta,
-                T* __restrict C,
+                C* __restrict dense_C,
                 int64_t              ldc,
                 rocsparse_order      order_C,
                 rocsparse_index_base base);
 
-template <typename T, typename I = rocsparse_int, typename J = rocsparse_int>
+template <typename T, typename I, typename J, typename A, typename B, typename C>
 void host_cscmm_batched(J                    M,
                         J                    N,
                         J                    K,
@@ -574,14 +576,14 @@ void host_cscmm_batched(J                    M,
                         T                    alpha,
                         const I*             csc_col_ptr_A,
                         const J*             csc_row_ind_A,
-                        const T*             csc_val_A,
-                        const T*             B,
+                        const A*             csc_val_A,
+                        const B*             dense_B,
                         int64_t              ldb,
                         J                    batch_count_B,
                         int64_t              batch_stride_B,
                         rocsparse_order      order_B,
                         T                    beta,
-                        T*                   C,
+                        C*                   dense_C,
                         int64_t              ldc,
                         J                    batch_count_C,
                         int64_t              batch_stride_C,
@@ -674,10 +676,10 @@ void host_bsrgeam_nnzb(rocsparse_direction  dir,
                        rocsparse_int        Mb,
                        rocsparse_int        Nb,
                        rocsparse_int        block_dim,
-                       T                    alpha,
+                       const T*             alpha,
                        const rocsparse_int* bsr_row_ptr_A,
                        const rocsparse_int* bsr_col_ind_A,
-                       T                    beta,
+                       const T*             beta,
                        const rocsparse_int* bsr_row_ptr_B,
                        const rocsparse_int* bsr_col_ind_B,
                        rocsparse_int*       bsr_row_ptr_C,
@@ -691,11 +693,11 @@ void host_bsrgeam(rocsparse_direction  dir,
                   rocsparse_int        Mb,
                   rocsparse_int        Nb,
                   rocsparse_int        block_dim,
-                  T                    alpha,
+                  const T*             alpha,
                   const rocsparse_int* bsr_row_ptr_A,
                   const rocsparse_int* bsr_col_ind_A,
                   const T*             bsr_val_A,
-                  T                    beta,
+                  const T*             beta,
                   const rocsparse_int* bsr_row_ptr_B,
                   const rocsparse_int* bsr_col_ind_B,
                   const T*             bsr_val_B,
@@ -706,34 +708,34 @@ void host_bsrgeam(rocsparse_direction  dir,
                   rocsparse_index_base base_B,
                   rocsparse_index_base base_C);
 
-template <typename T>
-void host_csrgeam_nnz(rocsparse_int        M,
-                      rocsparse_int        N,
-                      T                    alpha,
-                      const rocsparse_int* csr_row_ptr_A,
-                      const rocsparse_int* csr_col_ind_A,
-                      T                    beta,
-                      const rocsparse_int* csr_row_ptr_B,
-                      const rocsparse_int* csr_col_ind_B,
-                      rocsparse_int*       csr_row_ptr_C,
-                      rocsparse_int*       nnz_C,
+template <typename T, typename I, typename J>
+void host_csrgeam_nnz(J                    M,
+                      J                    N,
+                      const T*             alpha,
+                      const I*             csr_row_ptr_A,
+                      const J*             csr_col_ind_A,
+                      const T*             beta,
+                      const I*             csr_row_ptr_B,
+                      const J*             csr_col_ind_B,
+                      I*                   csr_row_ptr_C,
+                      I*                   nnz_C,
                       rocsparse_index_base base_A,
                       rocsparse_index_base base_B,
                       rocsparse_index_base base_C);
 
-template <typename T>
-void host_csrgeam(rocsparse_int        M,
-                  rocsparse_int        N,
-                  T                    alpha,
-                  const rocsparse_int* csr_row_ptr_A,
-                  const rocsparse_int* csr_col_ind_A,
+template <typename T, typename I, typename J>
+void host_csrgeam(J                    M,
+                  J                    N,
+                  const T*             alpha,
+                  const I*             csr_row_ptr_A,
+                  const J*             csr_col_ind_A,
                   const T*             csr_val_A,
-                  T                    beta,
-                  const rocsparse_int* csr_row_ptr_B,
-                  const rocsparse_int* csr_col_ind_B,
+                  const T*             beta,
+                  const I*             csr_row_ptr_B,
+                  const J*             csr_col_ind_B,
                   const T*             csr_val_B,
-                  const rocsparse_int* csr_row_ptr_C,
-                  rocsparse_int*       csr_col_ind_C,
+                  const I*             csr_row_ptr_C,
+                  J*                   csr_col_ind_C,
                   T*                   csr_val_C,
                   rocsparse_index_base base_A,
                   rocsparse_index_base base_B,
