@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -203,48 +203,22 @@ void testing_csr2bsr(const Arguments& arg)
 
     if(arg.timing)
     {
-        int number_cold_calls = 2;
-        int number_hot_calls  = arg.iters;
 
-        // Warm up
-        for(int iter = 0; iter < number_cold_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_csr2bsr<T>(handle,
-                                                       direction,
-                                                       dA.m,
-                                                       dA.n,
-                                                       csr_descr,
-                                                       dA.val,
-                                                       dA.ptr,
-                                                       dA.ind,
-                                                       block_dim,
-                                                       bsr_descr,
-                                                       dC.val,
-                                                       dC.ptr,
-                                                       dC.ind));
-        }
-
-        double gpu_time_used = get_time_us();
-
-        // Performance run
-        for(int iter = 0; iter < number_hot_calls; ++iter)
-        {
-            CHECK_ROCSPARSE_ERROR(rocsparse_csr2bsr<T>(handle,
-                                                       direction,
-                                                       dA.m,
-                                                       dA.n,
-                                                       csr_descr,
-                                                       dA.val,
-                                                       dA.ptr,
-                                                       dA.ind,
-                                                       block_dim,
-                                                       bsr_descr,
-                                                       dC.val,
-                                                       dC.ptr,
-                                                       dC.ind));
-        }
-
-        gpu_time_used = (get_time_us() - gpu_time_used) / number_hot_calls;
+        const double gpu_time_used = rocsparse_clients::run_benchmark(arg,
+                                                                      rocsparse_csr2bsr<T>,
+                                                                      handle,
+                                                                      direction,
+                                                                      dA.m,
+                                                                      dA.n,
+                                                                      csr_descr,
+                                                                      dA.val,
+                                                                      dA.ptr,
+                                                                      dA.ind,
+                                                                      block_dim,
+                                                                      bsr_descr,
+                                                                      dC.val,
+                                                                      dC.ptr,
+                                                                      dC.ind);
 
         double gbyte_count = csr2bsr_gbyte_count<T>(M, Mb, hA.nnz, *hbsr_nnzb, block_dim);
         double gpu_gbyte   = get_gpu_gbyte(gpu_time_used, gbyte_count);
