@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -769,6 +769,10 @@ void testing_csrgemm_reuse(const Arguments& arg)
             CHECK_ROCSPARSE_ERROR(rocsparse_csrgemm_symbolic(PARAMS_SYMBOLIC(d_A, d_B, d_C, d_D)));
             CHECK_ROCSPARSE_ERROR(
                 rocsparse_csrgemm_numeric<T>(PARAMS_NUMERIC(h_alpha, h_beta, d_A, d_B, d_C, d_D)));
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("dC pointer mode host", d_C);
+            }
             h_C.near_check(d_C);
         }
 
@@ -788,6 +792,10 @@ void testing_csrgemm_reuse(const Arguments& arg)
             CHECK_ROCSPARSE_ERROR(rocsparse_csrgemm_symbolic(PARAMS_SYMBOLIC(d_A, d_B, d_C, d_D)));
             CHECK_ROCSPARSE_ERROR(
                 rocsparse_csrgemm_numeric<T>(PARAMS_NUMERIC(d_alpha, d_beta, d_A, d_B, d_C, d_D)));
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("dC pointer mode device", d_C);
+            }
             h_C.near_check(d_C);
         }
     }
@@ -823,7 +831,7 @@ void testing_csrgemm_reuse(const Arguments& arg)
             }
         }
         ROCSPARSE_TIMER_OUT(gpu_num_time_used);
-        hipDeviceSynchronize();
+        CHECK_HIP_ERROR(hipDeviceSynchronize());
 
 #undef PARAMS
 #undef PARAM_NNZ
