@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -226,9 +226,28 @@ static void test_csrilu0_matrix(rocsparse_local_handle&    handle,
         // Check solution vector if no pivot has been found
         if(h_analysis_pivot_gold[0] == -1 && h_solve_pivot_gold[0] == -1)
         {
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save(
+                    "P pointer mode host", hcsr_val_1, "P pointer mode device", hcsr_val_2);
+            }
             hcsr_val_gold.near_check(hcsr_val_1);
             hcsr_val_gold.near_check(hcsr_val_2);
         }
+        else
+        {
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("Pivot analysis pointer mode host",
+                                                h_analysis_pivot_1,
+                                                "Pivot analysis pointer mode device",
+                                                h_analysis_pivot_2,
+                                                "Pivot solve pointer mode host",
+                                                h_solve_pivot_1,
+                                                "Pivot solve pointer mode device",
+                                                h_solve_pivot_2);
+            }
+        }
     }
 
     if(arg.timing)
@@ -320,21 +339,21 @@ static void test_csrilu0_matrix(rocsparse_local_handle&    handle,
 
         if(need_display)
         {
-            display_timing_info("M",
+            display_timing_info(display_key_t::M,
                                 M,
-                                "nnz",
+                                display_key_t::nnz_A,
                                 nnz,
-                                "pivot",
+                                display_key_t::pivot,
                                 pivot,
-                                "analysis policy",
+                                display_key_t::analysis_policy,
                                 rocsparse_analysis2string(apol),
-                                "solve policy",
+                                display_key_t::solve_policy,
                                 rocsparse_solve2string(spol),
-                                s_timing_info_bandwidth,
+                                display_key_t::bandwidth,
                                 gpu_gbyte,
-                                "analysis msec",
+                                display_key_t::analysis_time_ms,
                                 get_gpu_time_msec(gpu_analysis_time_used),
-                                s_timing_info_time,
+                                display_key_t::time_ms,
                                 get_gpu_time_msec(gpu_solve_time_used));
         }
     }
