@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -75,7 +75,8 @@ void testing_csrsv_bad_arg(const Arguments& arg)
 
     for(auto matrix_type : rocsparse_matrix_type_t::values)
     {
-        if(matrix_type != rocsparse_matrix_type_general)
+        if(matrix_type != rocsparse_matrix_type_general
+           && matrix_type != rocsparse_matrix_type_triangular)
         {
             CHECK_ROCSPARSE_ERROR(rocsparse_set_mat_type(descr, matrix_type));
             EXPECT_ROCSPARSE_STATUS(rocsparse_csrsv_buffer_size<T>(PARAMS_BUFFER_SIZE),
@@ -576,7 +577,7 @@ void testing_csrsv(const Arguments& arg)
                             gpu_gflops,
                             display_key_t::bandwidth,
                             gpu_gbyte,
-                            display_key_t::analysis_ms,
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
                             display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
