@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2020-2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -75,50 +75,14 @@ void testing_spgemm_csr_bad_arg(const Arguments& arg)
     rocsparse_datatype  compute_type = get_datatype<T>();
 
     // SpGEMM structures
-    rocsparse_local_spmat local_A(m,
-                                  k,
-                                  nnz_A,
-                                  csr_row_ptr_A,
-                                  csr_col_ind_A,
-                                  csr_val_A,
-                                  itype,
-                                  jtype,
-                                  base,
-                                  compute_type,
-                                  rocsparse_format_csr);
-    rocsparse_local_spmat local_B(k,
-                                  n,
-                                  nnz_B,
-                                  csr_row_ptr_B,
-                                  csr_col_ind_B,
-                                  csr_val_B,
-                                  itype,
-                                  jtype,
-                                  base,
-                                  compute_type,
-                                  rocsparse_format_csr);
-    rocsparse_local_spmat local_C(m,
-                                  n,
-                                  nnz_C,
-                                  csr_row_ptr_C,
-                                  csr_col_ind_C,
-                                  csr_val_C,
-                                  itype,
-                                  jtype,
-                                  base,
-                                  compute_type,
-                                  rocsparse_format_csr);
-    rocsparse_local_spmat local_D(m,
-                                  n,
-                                  nnz_D,
-                                  csr_row_ptr_D,
-                                  csr_col_ind_D,
-                                  csr_val_D,
-                                  itype,
-                                  jtype,
-                                  base,
-                                  compute_type,
-                                  rocsparse_format_csr);
+    rocsparse_local_spmat local_A(
+        m, k, nnz_A, csr_row_ptr_A, csr_col_ind_A, csr_val_A, itype, jtype, base, compute_type);
+    rocsparse_local_spmat local_B(
+        k, n, nnz_B, csr_row_ptr_B, csr_col_ind_B, csr_val_B, itype, jtype, base, compute_type);
+    rocsparse_local_spmat local_C(
+        m, n, nnz_C, csr_row_ptr_C, csr_col_ind_C, csr_val_C, itype, jtype, base, compute_type);
+    rocsparse_local_spmat local_D(
+        m, n, nnz_D, csr_row_ptr_D, csr_col_ind_D, csr_val_D, itype, jtype, base, compute_type);
 
     rocsparse_spmat_descr A = local_A;
     rocsparse_spmat_descr B = local_B;
@@ -492,6 +456,11 @@ void testing_spgemm_csr(const Arguments& arg)
                 CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
             }
 
+            if(ROCSPARSE_REPRODUCIBILITY)
+            {
+                rocsparse_reproducibility::save("dC pointer mode host", dC);
+            }
+
             //
             // Check
             //
@@ -552,6 +521,11 @@ void testing_spgemm_csr(const Arguments& arg)
                     CHECK_HIP_ERROR(rocsparse_hipFree(dbuffer));
                 }
 
+                if(ROCSPARSE_REPRODUCIBILITY)
+                {
+                    rocsparse_reproducibility::save("dC pointer mode device", dC);
+                }
+
                 if((!h_alpha_ptr || std::abs(*h_alpha_ptr) == ((I)std::abs(*h_alpha_ptr)))
                    && (!h_beta_ptr || std::abs(*h_beta_ptr) == ((I)std::abs(*h_beta_ptr))))
                 {
@@ -697,7 +671,7 @@ void testing_spgemm_csr(const Arguments& arg)
                             gpu_gflops,
                             display_key_t::bandwidth,
                             gpu_gbyte,
-                            display_key_t::analysis_ms,
+                            display_key_t::analysis_time_ms,
                             get_gpu_time_msec(gpu_analysis_time_used),
                             display_key_t::time_ms,
                             get_gpu_time_msec(gpu_solve_time_used));
