@@ -105,7 +105,7 @@ namespace rocsparse
             __threadfence_block();
 
             rocsparse::wfreduce_min<WFSIZE / ROW_BLOCKDIM>(&index_k);
-            next_k = __shfl(index_k, WFSIZE / ROW_BLOCKDIM - 1, WFSIZE / ROW_BLOCKDIM);
+            next_k = rocsparse::shfl(index_k, WFSIZE / ROW_BLOCKDIM - 1, WFSIZE / ROW_BLOCKDIM);
 
             if(found[wid] && lid == 0)
             {
@@ -113,7 +113,7 @@ namespace rocsparse
             }
 
             rocsparse::wfreduce_min<WFSIZE>(&min_block_col);
-            chunk_begin = __shfl(min_block_col, WFSIZE - 1, WFSIZE);
+            chunk_begin = rocsparse::shfl(min_block_col, WFSIZE - 1, WFSIZE);
 
             __threadfence_block();
         }
@@ -202,7 +202,8 @@ namespace rocsparse
             __syncthreads();
 
             rocsparse::wfreduce_min<BLOCKSIZE / ROW_BLOCKDIM>(&index_k);
-            next_k = __shfl(index_k, (BLOCKSIZE / ROW_BLOCKDIM) - 1, BLOCKSIZE / ROW_BLOCKDIM);
+            next_k = rocsparse::shfl(
+                index_k, (BLOCKSIZE / ROW_BLOCKDIM) - 1, BLOCKSIZE / ROW_BLOCKDIM);
 
             if(found && tid == 0)
             {
@@ -298,7 +299,7 @@ namespace rocsparse
             //
             // broadcast min_block_col_index from last thread in segment to all threads in segment
             //
-            min_block_col_index = __shfl(min_block_col_index, BLOCKSIZE - 1, BLOCKSIZE);
+            min_block_col_index = rocsparse::shfl(min_block_col_index, BLOCKSIZE - 1, BLOCKSIZE);
 
             block_col = min_block_col_index + 1;
 
@@ -410,7 +411,7 @@ namespace rocsparse
             __threadfence_block();
 
             rocsparse::wfreduce_min<WFSIZE / ROW_BLOCKDIM>(&index_k);
-            next_k = __shfl(index_k, (WFSIZE / ROW_BLOCKDIM) - 1, WFSIZE / ROW_BLOCKDIM);
+            next_k = rocsparse::shfl(index_k, (WFSIZE / ROW_BLOCKDIM) - 1, WFSIZE / ROW_BLOCKDIM);
 
             int offset = 0;
             if(table[wid])
@@ -446,7 +447,7 @@ namespace rocsparse
             block_row_begin += offset;
 
             rocsparse::wfreduce_min<WFSIZE>(&min_block_col);
-            chunk_begin = __shfl(min_block_col, WFSIZE - 1, WFSIZE);
+            chunk_begin = rocsparse::shfl(min_block_col, WFSIZE - 1, WFSIZE);
 
             __threadfence_block();
         }
@@ -535,7 +536,8 @@ namespace rocsparse
             __syncthreads();
 
             rocsparse::wfreduce_min<BLOCKSIZE / ROW_BLOCKDIM>(&index_k);
-            next_k = __shfl(index_k, (BLOCKSIZE / ROW_BLOCKDIM) - 1, BLOCKSIZE / ROW_BLOCKDIM);
+            next_k = rocsparse::shfl(
+                index_k, (BLOCKSIZE / ROW_BLOCKDIM) - 1, BLOCKSIZE / ROW_BLOCKDIM);
 
             int offset = 0;
             if(table)
@@ -687,10 +689,10 @@ namespace rocsparse
             }
 
             // broadcast CSR minimum column index from last thread in segment to all threads in segment
-            min_csr_col_index = __shfl(min_csr_col_index, BLOCKSIZE - 1, BLOCKSIZE);
+            min_csr_col_index = rocsparse::shfl(min_csr_col_index, BLOCKSIZE - 1, BLOCKSIZE);
 
             // broadcast nnzb_per_row from last thread in segment to all threads in segment
-            nnzb_per_row = __shfl(nnzb_per_row, BLOCKSIZE - 1, BLOCKSIZE);
+            nnzb_per_row = rocsparse::shfl(nnzb_per_row, BLOCKSIZE - 1, BLOCKSIZE);
 
             // Write BSR values
             for(rocsparse_int j = 0; j < rows_per_segment; j++)
