@@ -608,6 +608,12 @@ namespace rocsparse
                 (rocsparse::convert_data_array_compute_core<T, _Float16>)(p...));
             return rocsparse_status_success;
         }
+        case rocsparse_datatype_bf16_r:
+        {
+            RETURN_IF_ROCSPARSE_ERROR(
+                (rocsparse::convert_data_array_compute_core<T, rocsparse_bfloat16>)(p...));
+            return rocsparse_status_success;
+        }
         case rocsparse_datatype_f32_r:
         {
             RETURN_IF_ROCSPARSE_ERROR((rocsparse::convert_data_array_compute_core<T, float>)(p...));
@@ -670,13 +676,19 @@ namespace rocsparse
                 (rocsparse::convert_data_array_compute_dispatch<uint32_t>)(source_datatype_, p...));
             return rocsparse_status_success;
         }
-
         case rocsparse_datatype_f16_r:
         {
             RETURN_IF_ROCSPARSE_ERROR(
                 rocsparse::convert_data_array_compute_dispatch<_Float16>(source_datatype_, p...));
             return rocsparse_status_success;
         }
+        case rocsparse_datatype_bf16_r:
+        {
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::convert_data_array_compute_dispatch<rocsparse_bfloat16>(source_datatype_,
+                                                                                   p...));
+            return rocsparse_status_success;
+        }
         case rocsparse_datatype_f32_r:
         {
             RETURN_IF_ROCSPARSE_ERROR(
@@ -894,6 +906,7 @@ rocsparse_status rocsparse::dnvec_transfer_from(rocsparse_handle            hand
     case rocsparse_datatype_f16_r:
     case rocsparse_datatype_f32_r:
     case rocsparse_datatype_f64_r:
+    case rocsparse_datatype_bf16_r:
     {
         switch(source->data_type)
         {
@@ -913,6 +926,7 @@ rocsparse_status rocsparse::dnvec_transfer_from(rocsparse_handle            hand
         case rocsparse_datatype_f16_r:
         case rocsparse_datatype_f32_r:
         case rocsparse_datatype_f64_r:
+        case rocsparse_datatype_bf16_r:
         {
             break;
         }
