@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,13 +22,14 @@
  *
  * ************************************************************************ */
 
-#include "common.h"
-#include "utility.h"
+#include "rocsparse_common.hpp"
+#include "rocsparse_utility.hpp"
 
 #include "internal/conversion/rocsparse_coo2csr.h"
 #include "rocsparse_coo2csr.hpp"
 
 #include "coo2csr_device.h"
+#include "rocsparse_common.h"
 
 template <typename I, typename J>
 rocsparse_status rocsparse::coo2csr_core(rocsparse_handle     handle,
@@ -38,6 +39,8 @@ rocsparse_status rocsparse::coo2csr_core(rocsparse_handle     handle,
                                          I*                   csr_row_ptr,
                                          rocsparse_index_base idx_base)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Stream
     hipStream_t stream = handle->stream;
 
@@ -67,18 +70,14 @@ rocsparse_status rocsparse::coo2csr_template(rocsparse_handle     handle,
                                              I*                   csr_row_ptr,
                                              rocsparse_index_base idx_base)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     if(m == 0)
     {
         if(csr_row_ptr != nullptr)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<1>),
-                                               dim3(1),
-                                               dim3(1),
-                                               0,
-                                               handle->stream,
-                                               m + 1,
-                                               csr_row_ptr,
-                                               static_cast<I>(idx_base));
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::valset(handle, m + 1, static_cast<I>(idx_base), csr_row_ptr));
         }
         return rocsparse_status_success;
     }
@@ -95,6 +94,8 @@ rocsparse_status rocsparse::coo2csr_impl(rocsparse_handle     handle,
                                          I*                   csr_row_ptr,
                                          rocsparse_index_base idx_base)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Logging
     rocsparse::log_trace(handle,
                          "rocsparse_coo2csr",
@@ -160,11 +161,15 @@ extern "C" rocsparse_status rocsparse_coo2csr(rocsparse_handle     handle,
                                               rocsparse_index_base idx_base)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(
         rocsparse::coo2csr_impl(handle, coo_row_ind, nnz, m, csr_row_ptr, idx_base));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
