@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,14 +22,15 @@
  *
  * ************************************************************************ */
 
-#include "control.h"
 #include "internal/conversion/rocsparse_csr2bsr.h"
+#include "rocsparse_common.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csr2bsr.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "csr2bsr_nnz_device.h"
 
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 /*
  * ===========================================================================
@@ -85,6 +86,8 @@ rocsparse_status rocsparse::csr2bsr_nnz_quickreturn(rocsparse_handle          ha
                                                     I*                        bsr_row_ptr,
                                                     I*                        bsr_nnz)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     if(m == 0 || n == 0)
     {
         if(bsr_nnz != nullptr)
@@ -102,14 +105,8 @@ rocsparse_status rocsparse::csr2bsr_nnz_quickreturn(rocsparse_handle          ha
         if(bsr_row_ptr != nullptr)
         {
             const J mb = (m + block_dim - 1) / block_dim;
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                               dim3(((mb + 1) - 1) / 256 + 1),
-                                               dim3(256),
-                                               0,
-                                               handle->stream,
-                                               (mb + 1),
-                                               bsr_row_ptr,
-                                               static_cast<I>(bsr_descr->base));
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::valset(handle, mb + 1, static_cast<I>(bsr_descr->base), bsr_row_ptr));
         }
 
         return rocsparse_status_success;
@@ -133,6 +130,7 @@ namespace rocsparse
                                                  I*                        bsr_row_ptr, //9
                                                  I*                        bsr_nnz) //10
     {
+        ROCSPARSE_ROUTINE_TRACE;
 
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_ENUM(1, dir);
@@ -206,6 +204,7 @@ rocsparse_status rocsparse::csr2bsr_nnz_core(rocsparse_handle          handle,
                                              I*                        bsr_row_ptr,
                                              I*                        bsr_nnz)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_ENUM(1, dir);
@@ -244,14 +243,8 @@ rocsparse_status rocsparse::csr2bsr_nnz_core(rocsparse_handle          handle,
 
         if(bsr_row_ptr != nullptr)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                               dim3(((mb + 1) - 1) / 256 + 1),
-                                               dim3(256),
-                                               0,
-                                               handle->stream,
-                                               (mb + 1),
-                                               bsr_row_ptr,
-                                               static_cast<I>(bsr_descr->base));
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::valset(handle, mb + 1, static_cast<I>(bsr_descr->base), bsr_row_ptr));
         }
 
         return rocsparse_status_success;
@@ -405,10 +398,9 @@ rocsparse_status rocsparse::csr2bsr_nnz_core(rocsparse_handle          handle,
     }
 
     // Perform inclusive scan on bsr row pointer array
-    auto   op = rocprim::plus<I>();
     size_t temp_storage_size_bytes;
-    RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(
-        nullptr, temp_storage_size_bytes, bsr_row_ptr, bsr_row_ptr, mb + 1, op, handle->stream));
+    RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::inclusive_scan_buffer_size<I, I>(
+        handle, mb + 1, &temp_storage_size_bytes)));
 
     bool  temp_alloc       = false;
     void* temp_storage_ptr = nullptr;
@@ -424,13 +416,8 @@ rocsparse_status rocsparse::csr2bsr_nnz_core(rocsparse_handle          handle,
         temp_alloc = true;
     }
 
-    RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(temp_storage_ptr,
-                                                temp_storage_size_bytes,
-                                                bsr_row_ptr,
-                                                bsr_row_ptr,
-                                                mb + 1,
-                                                op,
-                                                handle->stream));
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(
+        handle, bsr_row_ptr, bsr_row_ptr, mb + 1, temp_storage_size_bytes, temp_storage_ptr));
 
     if(temp_alloc)
     {
@@ -501,6 +488,8 @@ namespace rocsparse
     template <typename... P>
     static rocsparse_status csr2bsr_nnz_impl(P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         rocsparse::log_trace("rocsparse_csr2bsr_nnz", p...);
         const rocsparse_status status = rocsparse::csr2bsr_nnz_checkarg(p...);
         if(status != rocsparse_status_continue)
@@ -526,6 +515,8 @@ extern "C" rocsparse_status rocsparse_csr2bsr_nnz(rocsparse_handle          hand
                                                   rocsparse_int*            bsr_nnz)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::csr2bsr_nnz_impl(handle,
                                                           dir,
                                                           m,
@@ -538,8 +529,10 @@ try
                                                           bsr_row_ptr,
                                                           bsr_nnz));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
