@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,16 +23,17 @@
  * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_csr2csc.h"
-#include "common.h"
-#include "control.h"
+#include "rocsparse_common.hpp"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csr2csc.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "csr2csc_device.h"
+#include "rocsparse_common.h"
 #include "rocsparse_coo2csr.hpp"
 #include "rocsparse_csr2coo.hpp"
 #include "rocsparse_identity.hpp"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 template <typename I, typename J, typename T>
 rocsparse_status rocsparse::csr2csc_core(rocsparse_handle     handle,
@@ -50,6 +51,8 @@ rocsparse_status rocsparse::csr2csc_core(rocsparse_handle     handle,
                                          rocsparse_index_base idx_base,
                                          void*                temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Stream
     hipStream_t stream = handle->stream;
 
@@ -85,16 +88,16 @@ rocsparse_status rocsparse::csr2csc_core(rocsparse_handle     handle,
         // Create row indices
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csr2coo_core(
             handle, csr_row_ptr_begin, csr_row_ptr_end, nnz, m, csc_row_ind, idx_base));
+
         // Stable sort COO by columns
-        rocprim::double_buffer<J> keys(tmp_work1, tmp_perm);
-        rocprim::double_buffer<J> vals(csc_row_ind, tmp_work2);
+        rocsparse::primitives::double_buffer<J> keys(tmp_work1, tmp_perm);
+        rocsparse::primitives::double_buffer<J> vals(csc_row_ind, tmp_work2);
 
         size_t size = 0;
-
-        RETURN_IF_HIP_ERROR(
-            rocprim::radix_sort_pairs(nullptr, size, keys, vals, nnz, startbit, endbit, stream));
-        RETURN_IF_HIP_ERROR(rocprim::radix_sort_pairs(
-            tmp_rocprim, size, keys, vals, nnz, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::radix_sort_pairs_buffer_size<J, J>(
+            handle, nnz, startbit, endbit, &size)));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::radix_sort_pairs(
+            handle, keys, vals, nnz, startbit, endbit, size, tmp_rocprim));
 
         // Create column pointers
         RETURN_IF_ROCSPARSE_ERROR(
@@ -127,15 +130,14 @@ rocsparse_status rocsparse::csr2csc_core(rocsparse_handle     handle,
             rocsparse::create_identity_permutation_core(handle, nnz, tmp_perm));
 
         // Stable sort COO by columns
-        rocprim::double_buffer<J> keys(tmp_work1, csc_row_ind);
-        rocprim::double_buffer<I> vals(tmp_perm, tmp_work2);
+        rocsparse::primitives::double_buffer<J> keys(tmp_work1, csc_row_ind);
+        rocsparse::primitives::double_buffer<I> vals(tmp_perm, tmp_work2);
 
         size_t size = 0;
-
-        RETURN_IF_HIP_ERROR(
-            rocprim::radix_sort_pairs(nullptr, size, keys, vals, nnz, startbit, endbit, stream));
-        RETURN_IF_HIP_ERROR(rocprim::radix_sort_pairs(
-            tmp_rocprim, size, keys, vals, nnz, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::radix_sort_pairs_buffer_size<J, I>(
+            handle, nnz, startbit, endbit, &size)));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::radix_sort_pairs(
+            handle, keys, vals, nnz, startbit, endbit, size, tmp_rocprim));
 
         // Create column pointers
         RETURN_IF_ROCSPARSE_ERROR(
@@ -181,24 +183,16 @@ rocsparse_status rocsparse::csr2csc_template(rocsparse_handle     handle,
                                              rocsparse_index_base idx_base,
                                              void*                temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Quick return if possible
     if(m == 0 || n == 0 || nnz == 0)
     {
-        return rocsparse_status_success;
-    }
-
-    if(nnz == 0)
-    {
-        RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                           dim3(n / 256 + 1),
-                                           dim3(256),
-                                           0,
-                                           handle->stream,
-                                           (n + 1),
-                                           csc_col_ptr,
-                                           static_cast<I>(idx_base));
-
+        if(nnz == 0 && csc_col_ptr != nullptr)
+        {
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::valset(handle, n + 1, static_cast<I>(idx_base), csc_col_ptr));
+        }
         return rocsparse_status_success;
     }
 
@@ -239,6 +233,7 @@ rocsparse_status rocsparse::csr2csc_impl(rocsparse_handle     handle, //0
                                          rocsparse_index_base idx_base, //11
                                          void*                temp_buffer) //12
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -278,14 +273,8 @@ rocsparse_status rocsparse::csr2csc_impl(rocsparse_handle     handle, //0
     {
         if(nnz == 0 && csc_col_ptr != nullptr)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                               dim3(n / 256 + 1),
-                                               dim3(256),
-                                               0,
-                                               handle->stream,
-                                               (n + 1),
-                                               csc_col_ptr,
-                                               static_cast<I>(idx_base));
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::valset(handle, n + 1, static_cast<I>(idx_base), csc_col_ptr));
         }
         return rocsparse_status_success;
     }
@@ -411,16 +400,13 @@ rocsparse_status rocsparse::csr2csc_buffer_size_core(rocsparse_handle handle,
                                                      rocsparse_action copy_values,
                                                      size_t*          buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
-    hipStream_t stream = handle->stream;
+    uint32_t startbit = 0;
+    uint32_t endbit   = rocsparse::clz(n);
 
-    // Determine rocprim buffer size
-    J* ptr = reinterpret_cast<J*>(buffer_size);
-
-    rocprim::double_buffer<J> dummy(ptr, ptr);
-
-    RETURN_IF_HIP_ERROR(
-        rocprim::radix_sort_pairs(nullptr, *buffer_size, dummy, dummy, nnz, 0, 32, stream));
+    RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::radix_sort_pairs_buffer_size<J, J>(
+        handle, nnz, startbit, endbit, buffer_size)));
 
     *buffer_size = ((*buffer_size - 1) / 256 + 1) * 256;
 
@@ -443,6 +429,8 @@ rocsparse_status rocsparse::csr2csc_buffer_size_template(rocsparse_handle handle
                                                          rocsparse_action copy_values,
                                                          size_t*          buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Quick return if possible
     if(m == 0 || n == 0 || nnz == 0)
     {
@@ -465,6 +453,8 @@ rocsparse_status rocsparse::csr2csc_buffer_size_impl(rocsparse_handle handle,
                                                      rocsparse_action copy_values,
                                                      size_t*          buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
 
     // Logging
@@ -544,14 +534,18 @@ extern "C" rocsparse_status rocsparse_csr2csc_buffer_size(rocsparse_handle     h
                                                           size_t*              buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::csr2csc_buffer_size_impl(
         handle, m, n, nnz, csr_row_ptr, csr_col_ind, copy_values, buffer_size));
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 #define CIMPL(NAME, T)                                                   \
     extern "C" rocsparse_status NAME(rocsparse_handle     handle,        \
@@ -569,6 +563,7 @@ catch(...)
                                      void*                temp_buffer)   \
     try                                                                  \
     {                                                                    \
+        ROCSPARSE_ROUTINE_TRACE;                                         \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csr2csc_impl(handle,        \
                                                           m,             \
                                                           n,             \
