@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,13 +26,14 @@
 #include "internal/conversion/rocsparse_csr2bsr.h"
 #include "internal/conversion/rocsparse_csr2coo.h"
 
-#include "control.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csr2bsr.hpp"
 #include "rocsparse_csr2gebsr.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "csr2gebsr_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_common.h"
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -107,6 +108,7 @@ namespace rocsparse
                                                             rocsparse_int             row_block_dim,
                                                             rocsparse_int             col_block_dim)
     {
+        ROCSPARSE_ROUTINE_TRACE;
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_internal_error);
     }
 
@@ -133,6 +135,8 @@ namespace rocsparse
                                                             rocsparse_int             row_block_dim,
                                                             rocsparse_int             col_block_dim)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         hipStream_t stream = handle->stream;
 
         launch_csr2gebsr_block_per_row_multipass_kernel(256, 64, 64);
@@ -154,6 +158,8 @@ rocsparse_status rocsparse::csr2gebsr_buffer_size_template(rocsparse_handle
                                                            rocsparse_int        col_block_dim, //9
                                                            size_t*              buffer_size) //10
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     //
     // Logging
     //
@@ -242,6 +248,7 @@ rocsparse_status rocsparse::csr2gebsr_template(rocsparse_handle          handle,
                                                rocsparse_int             col_block_dim, //13
                                                void*                     temp_buffer) //14
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     rocsparse::log_trace(handle,
                          rocsparse::replaceX<T>("rocsparse_Xcsr2gebsr"),
@@ -783,6 +790,8 @@ extern "C" rocsparse_status rocsparse_csr2gebsr_nnz(rocsparse_handle          ha
                                                     void*                     temp_buffer) //12
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Logging
     rocsparse::log_trace(handle,
                          "rocsparse_csr2gebsr_nnz",
@@ -841,14 +850,8 @@ try
 
         if(bsr_row_ptr != nullptr)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                               dim3(((mb + 1) - 1) / 256 + 1),
-                                               dim3(256),
-                                               0,
-                                               handle->stream,
-                                               (mb + 1),
-                                               bsr_row_ptr,
-                                               static_cast<rocsparse_int>(bsr_descr->base));
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                handle, mb + 1, static_cast<rocsparse_int>(bsr_descr->base), bsr_row_ptr));
         }
 
         return rocsparse_status_success;
@@ -904,15 +907,10 @@ try
                                            col_block_dim);
 
         // Perform inclusive scan on bsr row pointer array
-        auto   op = rocprim::plus<rocsparse_int>();
         size_t temp_storage_size_bytes;
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(nullptr,
-                                                    temp_storage_size_bytes,
-                                                    bsr_row_ptr,
-                                                    bsr_row_ptr,
-                                                    mb + 1,
-                                                    op,
-                                                    handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::inclusive_scan_buffer_size<rocsparse_int, rocsparse_int>(
+                handle, mb + 1, &temp_storage_size_bytes)));
 
         bool  temp_alloc       = false;
         void* temp_storage_ptr = nullptr;
@@ -928,13 +926,8 @@ try
             temp_alloc = true;
         }
 
-        RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(temp_storage_ptr,
-                                                    temp_storage_size_bytes,
-                                                    bsr_row_ptr,
-                                                    bsr_row_ptr,
-                                                    mb + 1,
-                                                    op,
-                                                    handle->stream));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(
+            handle, bsr_row_ptr, bsr_row_ptr, mb + 1, temp_storage_size_bytes, temp_storage_ptr));
 
         if(temp_alloc)
         {
@@ -1115,10 +1108,10 @@ try
     }
 
     // Perform inclusive scan on bsr row pointer array
-    auto   op = rocprim::plus<rocsparse_int>();
     size_t temp_storage_size_bytes;
-    RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(
-        nullptr, temp_storage_size_bytes, bsr_row_ptr, bsr_row_ptr, mb + 1, op, handle->stream));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::inclusive_scan_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, mb + 1, &temp_storage_size_bytes)));
 
     bool  temp_alloc       = false;
     void* temp_storage_ptr = nullptr;
@@ -1134,13 +1127,8 @@ try
         temp_alloc = true;
     }
 
-    RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(temp_storage_ptr,
-                                                temp_storage_size_bytes,
-                                                bsr_row_ptr,
-                                                bsr_row_ptr,
-                                                mb + 1,
-                                                op,
-                                                handle->stream));
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(
+        handle, bsr_row_ptr, bsr_row_ptr, mb + 1, temp_storage_size_bytes, temp_storage_ptr));
 
     if(temp_alloc)
     {
@@ -1176,11 +1164,13 @@ try
     }
 
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 //
 // C INTERFACE
@@ -1199,6 +1189,7 @@ catch(...)
                                         size_t*                   buffer_size)             \
     try                                                                                    \
     {                                                                                      \
+        ROCSPARSE_ROUTINE_TRACE;                                                           \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csr2gebsr_buffer_size_template(handle,        \
                                                                             dir,           \
                                                                             m,             \
@@ -1235,6 +1226,7 @@ catch(...)
                                                                                            \
     try                                                                                    \
     {                                                                                      \
+        ROCSPARSE_ROUTINE_TRACE;                                                           \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csr2gebsr_template(handle,                    \
                                                                 dir,                       \
                                                                 m,                         \
