@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,12 +22,12 @@
  *
  * ************************************************************************ */
 #include "internal/conversion/rocsparse_csrsort.h"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
-#include "control.h"
 #include "csrsort_device.h"
+#include "rocsparse_control.hpp"
 
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 extern "C" rocsparse_status rocsparse_csrsort_buffer_size(rocsparse_handle     handle,
                                                           rocsparse_int        m,
@@ -38,6 +38,8 @@ extern "C" rocsparse_status rocsparse_csrsort_buffer_size(rocsparse_handle     h
                                                           size_t*              buffer_size)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     // Logging
     rocsparse::log_trace(handle,
                          "rocsparse_csrsort_buffer_size",
@@ -62,29 +64,20 @@ try
         return rocsparse_status_success;
     }
 
-    // Stream
-    hipStream_t stream = handle->stream;
-
-    rocsparse_int*                        ptr = reinterpret_cast<rocsparse_int*>(buffer_size);
-    rocprim::double_buffer<rocsparse_int> dummy(ptr, ptr);
-
     uint32_t startbit = 0;
     uint32_t endbit   = rocsparse::clz(n);
 
-    using config
-        = rocprim::segmented_radix_sort_config<7,
-                                               4,
-                                               rocprim::kernel_config<256, 16>,
-                                               rocprim::WarpSortConfig<8, 8, 256, 5, 16, 16, 256>,
-                                               1>;
-
     // We do not know if sort_pairs or sort_keys will be called, so use the largest buffer between the two
     size_t size1;
     size_t size2;
-    RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_pairs<config>(
-        nullptr, size1, dummy, dummy, nnz, m, ptr, ptr + 1, startbit, endbit, stream));
-    RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys<config>(
-        nullptr, size2, dummy, nnz, m, ptr, ptr + 1, startbit, endbit, stream));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::
+             segmented_radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int, rocsparse_int>(
+                 handle, nnz, m, startbit, endbit, &size1)));
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::segmented_radix_sort_keys_buffer_size<rocsparse_int, rocsparse_int>(
+            handle, nnz, m, startbit, endbit, &size2)));
+
     *buffer_size = rocsparse::max(size1, size2);
 
     *buffer_size = ((*buffer_size - 1) / 256 + 1) * 256;
@@ -100,11 +93,13 @@ try
     *buffer_size += ((sizeof(rocsparse_int) * m) / 256 + 1) * 256;
 
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
 
 extern "C" rocsparse_status rocsparse_csrsort(rocsparse_handle          handle,
                                               rocsparse_int             m,
@@ -117,6 +112,7 @@ extern "C" rocsparse_status rocsparse_csrsort(rocsparse_handle          handle,
                                               void*                     temp_buffer)
 try
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -152,37 +148,21 @@ try
     uint32_t endbit   = rocsparse::clz(n);
     size_t   size;
 
-    using config
-        = rocprim::segmented_radix_sort_config<7,
-                                               4,
-                                               rocprim::kernel_config<256, 16>,
-                                               rocprim::WarpSortConfig<8, 8, 256, 5, 16, 16, 256>,
-                                               1>;
-
     if(perm != nullptr)
     {
         // Sort pairs, if permutation vector is present
-        rocprim::double_buffer<rocsparse_int> dummy(csr_col_ind, perm);
-
-        RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_pairs<config>(nullptr,
-                                                                        size,
-                                                                        dummy,
-                                                                        dummy,
-                                                                        nnz,
-                                                                        m,
-                                                                        csr_row_ptr,
-                                                                        csr_row_ptr + 1,
-                                                                        startbit,
-                                                                        endbit,
-                                                                        stream));
+        RETURN_IF_ROCSPARSE_ERROR((
+            rocsparse::primitives::
+                segmented_radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int, rocsparse_int>(
+                    handle, nnz, m, startbit, endbit, &size)));
     }
     else
     {
         // Sort keys, if no permutation vector is present
-        rocprim::double_buffer<rocsparse_int> dummy(csr_col_ind, csr_col_ind);
-
-        RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys<config>(
-            nullptr, size, dummy, nnz, m, csr_row_ptr, csr_row_ptr + 1, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::segmented_radix_sort_keys_buffer_size<rocsparse_int,
+                                                                          rocsparse_int>(
+                handle, nnz, m, startbit, endbit, &size)));
     }
 
     // Temporary buffer entry points
@@ -229,11 +209,11 @@ try
     if(perm != nullptr)
     {
         // Sort by pairs, if permutation vector is present
-        rocprim::double_buffer<rocsparse_int> keys(csr_col_ind, tmp_cols);
-        rocprim::double_buffer<rocsparse_int> vals(perm, tmp_perm);
+        rocsparse::primitives::double_buffer<rocsparse_int> keys(csr_col_ind, tmp_cols);
+        rocsparse::primitives::double_buffer<rocsparse_int> vals(perm, tmp_perm);
 
-        RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_pairs<config>(
-            tmp_rocprim, size, keys, vals, nnz, m, offsets, offsets + 1, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::segmented_radix_sort_pairs(
+            handle, keys, vals, nnz, m, offsets, offsets + 1, startbit, endbit, size, tmp_rocprim));
 
         if(keys.current() != csr_col_ind)
         {
@@ -255,10 +235,10 @@ try
     else
     {
         // Sort by keys, if no permutation vector is present
-        rocprim::double_buffer<rocsparse_int> keys(csr_col_ind, tmp_cols);
+        rocsparse::primitives::double_buffer<rocsparse_int> keys(csr_col_ind, tmp_cols);
 
-        RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys<config>(
-            tmp_rocprim, size, keys, nnz, m, offsets, offsets + 1, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::segmented_radix_sort_keys(
+            handle, keys, nnz, m, offsets, offsets + 1, startbit, endbit, size, tmp_rocprim));
 
         if(keys.current() != csr_col_ind)
         {
@@ -270,8 +250,10 @@ try
         }
     }
     return rocsparse_status_success;
+    // LCOV_EXCL_START
 }
 catch(...)
 {
     RETURN_ROCSPARSE_EXCEPTION();
 }
+// LCOV_EXCL_STOP
