@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -39,6 +39,8 @@ namespace rocsparse
                                         I*                        csx_row_col_ptr,
                                         J*                        csx_col_row_ind)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         if(0 == m || 0 == n)
         {
             return rocsparse_status_success;
@@ -145,6 +147,8 @@ namespace rocsparse
         }
         }
 
+        // LCOV_EXCL_START
         return rocsparse_status_invalid_value;
+        // LCOV_EXCL_STOP
     }
 }
