@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,11 +23,12 @@
  * ************************************************************************ */
 #pragma once
 
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
-#include "control.h"
+#include "rocsparse_common.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_dense2csx.hpp"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -44,6 +45,8 @@ namespace rocsparse
                                         J*                        csx_col_row_ind_A, //9
                                         rocsparse_order           order) //10
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         static constexpr bool is_row_oriented = (rocsparse_direction_row == DIRA);
 
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
@@ -60,14 +63,8 @@ namespace rocsparse
             {
                 J dimdir = is_row_oriented ? m : n;
 
-                RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                                   dim3(dimdir / 256 + 1),
-                                                   dim3(256),
-                                                   0,
-                                                   handle->stream,
-                                                   (dimdir + 1),
-                                                   csx_row_col_ptr_A,
-                                                   static_cast<I>(descr->base));
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                    handle, dimdir + 1, static_cast<I>(descr->base), csx_row_col_ptr_A));
             }
 
             return rocsparse_status_success;
@@ -132,6 +129,8 @@ namespace rocsparse
                                     I*                        csx_row_col_ptr_A,
                                     J*                        csx_col_row_ind_A)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         static constexpr bool is_row_oriented = (rocsparse_direction_row == DIRA);
 
         //
@@ -182,13 +181,8 @@ namespace rocsparse
 
             size_t temp_storage_bytes = 0;
             // Obtain rocprim buffer size
-            RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(nullptr,
-                                                        temp_storage_bytes,
-                                                        csx_row_col_ptr_A,
-                                                        csx_row_col_ptr_A,
-                                                        dimdir + 1,
-                                                        rocprim::plus<I>(),
-                                                        handle->stream));
+            RETURN_IF_ROCSPARSE_ERROR((rocsparse::primitives::inclusive_scan_buffer_size<I, I>(
+                handle, dimdir + 1, &temp_storage_bytes)));
 
             // Get rocprim buffer
             bool  d_temp_alloc;
@@ -208,13 +202,13 @@ namespace rocsparse
             }
 
             // Perform actual inclusive sum
-            RETURN_IF_HIP_ERROR(rocprim::inclusive_scan(d_temp_storage,
-                                                        temp_storage_bytes,
-                                                        csx_row_col_ptr_A,
-                                                        csx_row_col_ptr_A,
-                                                        dimdir + 1,
-                                                        rocprim::plus<I>(),
-                                                        handle->stream));
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::inclusive_scan(handle,
+                                                                            csx_row_col_ptr_A,
+                                                                            csx_row_col_ptr_A,
+                                                                            dimdir + 1,
+                                                                            temp_storage_bytes,
+                                                                            d_temp_storage));
+
             // Free rocprim buffer, if allocated
             if(d_temp_alloc == true)
             {
