@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,7 +23,7 @@
 #include "rocsparse_gcoo2csr.hpp"
 #include "rocsparse_convert_array.hpp"
 #include "rocsparse_coo2csr.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 rocsparse_status rocsparse::gcoo2csr(rocsparse_handle     handle_,
                                      rocsparse_indextype  source_row_type_,
@@ -34,6 +34,8 @@ rocsparse_status rocsparse::gcoo2csr(rocsparse_handle     handle_,
                                      void*                target_row_,
                                      rocsparse_index_base idx_base_)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
 #define DO(SROW, TROW)                                                          \
     do                                                                          \
     {                                                                           \
@@ -64,7 +66,9 @@ rocsparse_status rocsparse::gcoo2csr(rocsparse_handle     handle_,
         case rocsparse_indextype_i64:
             DO(int32_t, int64_t);
         }
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     case rocsparse_indextype_i64:
@@ -78,11 +82,15 @@ rocsparse_status rocsparse::gcoo2csr(rocsparse_handle     handle_,
         case rocsparse_indextype_i64:
             DO(int64_t, int64_t);
         }
+        // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
     }
 #undef DO
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 rocsparse_status rocsparse::spmat_coo2csr_buffer_size(rocsparse_handle            handle,
@@ -90,6 +98,8 @@ rocsparse_status rocsparse::spmat_coo2csr_buffer_size(rocsparse_handle
                                                       rocsparse_spmat_descr       target_,
                                                       size_t*                     buffer_size_)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     buffer_size_[0] = 0;
     return rocsparse_status_success;
 }
@@ -100,6 +110,8 @@ rocsparse_status rocsparse::spmat_coo2csr(rocsparse_handle            handle,
                                           size_t                      buffer_size_,
                                           void*                       buffer_)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse::gcoo2csr(handle,
                                                   source_->row_type,
                                                   source_->const_row_data,
