@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,16 +23,17 @@
  * ************************************************************************ */
 
 #include "internal/conversion/rocsparse_gebsr2gebsc.h"
-#include "common.h"
-#include "control.h"
 #include "internal/conversion/rocsparse_coo2csr.h"
 #include "internal/conversion/rocsparse_csr2coo.h"
 #include "internal/conversion/rocsparse_inverse_permutation.h"
+#include "rocsparse_common.hpp"
+#include "rocsparse_control.hpp"
 #include "rocsparse_gebsr2gebsc.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "gebsr2gebsc_device.h"
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_common.h"
+#include "rocsparse_primitives.hpp"
 
 namespace rocsparse
 {
@@ -52,19 +53,15 @@ namespace rocsparse
                                                     rocsparse_index_base idx_base,
                                                     void*                temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Quick return if possible
         if(mb == 0 || nb == 0)
         {
             if(bsc_col_ptr != nullptr)
             {
-                RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                                   dim3(nb / 256 + 1),
-                                                   dim3(256),
-                                                   0,
-                                                   handle->stream,
-                                                   (nb + 1),
-                                                   bsc_col_ptr,
-                                                   static_cast<rocsparse_int>(idx_base));
+                RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                    handle, nb + 1, static_cast<rocsparse_int>(idx_base), bsc_col_ptr));
             }
 
             return rocsparse_status_success;
@@ -72,14 +69,8 @@ namespace rocsparse
 
         if(nnzb == 0)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<256>),
-                                               dim3(nb / 256 + 1),
-                                               dim3(256),
-                                               0,
-                                               handle->stream,
-                                               (nb + 1),
-                                               bsc_col_ptr,
-                                               static_cast<rocsparse_int>(idx_base));
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse::valset(
+                handle, nb + 1, static_cast<rocsparse_int>(idx_base), bsc_col_ptr));
 
             return rocsparse_status_success;
         }
@@ -104,6 +95,7 @@ rocsparse_status rocsparse::gebsr2gebsc_template(rocsparse_handle     handle, //
                                                  rocsparse_index_base idx_base, //13
                                                  void*                temp_buffer) //14
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Logging
     rocsparse::log_trace(handle,
@@ -205,15 +197,15 @@ rocsparse_status rocsparse::gebsr2gebsc_template(rocsparse_handle     handle, //
         RETURN_IF_ROCSPARSE_ERROR(
             rocsparse_csr2coo(handle, bsr_row_ptr, nnzb, mb, bsc_row_ind, idx_base));
         // Stable sort COO by columns
-        rocprim::double_buffer<rocsparse_int> keys(tmp_work1, tmp_perm);
-        rocprim::double_buffer<rocsparse_int> vals(bsc_row_ind, tmp_work2);
+        rocsparse::primitives::double_buffer<rocsparse_int> keys(tmp_work1, tmp_perm);
+        rocsparse::primitives::double_buffer<rocsparse_int> vals(bsc_row_ind, tmp_work2);
 
         size_t size = 0;
-
-        RETURN_IF_HIP_ERROR(
-            rocprim::radix_sort_pairs(nullptr, size, keys, vals, nnzb, startbit, endbit, stream));
-        RETURN_IF_HIP_ERROR(rocprim::radix_sort_pairs(
-            tmp_rocprim, size, keys, vals, nnzb, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int>(
+                handle, nnzb, startbit, endbit, &size)));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::radix_sort_pairs(
+            handle, keys, vals, nnzb, startbit, endbit, size, tmp_rocprim));
 
         // Create column pointers
         RETURN_IF_ROCSPARSE_ERROR(
@@ -237,15 +229,15 @@ rocsparse_status rocsparse::gebsr2gebsc_template(rocsparse_handle     handle, //
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_create_identity_permutation(handle, nnzb, tmp_perm));
 
         // Stable sort COO by columns
-        rocprim::double_buffer<rocsparse_int> keys(tmp_work1, bsc_row_ind);
-        rocprim::double_buffer<rocsparse_int> vals(tmp_perm, tmp_work2);
+        rocsparse::primitives::double_buffer<rocsparse_int> keys(tmp_work1, bsc_row_ind);
+        rocsparse::primitives::double_buffer<rocsparse_int> vals(tmp_perm, tmp_work2);
 
         size_t size = 0;
-
-        RETURN_IF_HIP_ERROR(
-            rocprim::radix_sort_pairs(nullptr, size, keys, vals, nnzb, startbit, endbit, stream));
-        RETURN_IF_HIP_ERROR(rocprim::radix_sort_pairs(
-            tmp_rocprim, size, keys, vals, nnzb, startbit, endbit, stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int>(
+                handle, nnzb, startbit, endbit, &size)));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::radix_sort_pairs(
+            handle, keys, vals, nnzb, startbit, endbit, size, tmp_rocprim));
 
         // Create column pointers
         RETURN_IF_ROCSPARSE_ERROR(
@@ -291,6 +283,8 @@ namespace rocsparse
                                                                 rocsparse_int    col_block_dim,
                                                                 size_t*          p_buffer_size)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Quick return if possible
         if(mb == 0 || nb == 0)
         {
@@ -312,6 +306,8 @@ namespace rocsparse
                                                       rocsparse_int        col_block_dim, //8
                                                       size_t*              p_buffer_size) //9
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         // Logging
         rocsparse::log_trace(handle,
                              "rocsparse_gebsr2gebsc_buffer_size",
@@ -358,15 +354,12 @@ namespace rocsparse
         ROCSPARSE_CHECKARG_ARRAY(5, mb, bsr_row_ptr);
         ROCSPARSE_CHECKARG_ARRAY(6, nnzb, bsr_col_ind);
 
-        hipStream_t stream = handle->stream;
+        uint32_t startbit = 0;
+        uint32_t endbit   = rocsparse::clz(nb);
 
-        // Determine rocprim buffer size
-        rocsparse_int* ptr = reinterpret_cast<rocsparse_int*>(p_buffer_size);
-
-        rocprim::double_buffer<rocsparse_int> dummy(ptr, ptr);
-
-        RETURN_IF_HIP_ERROR(
-            rocprim::radix_sort_pairs(nullptr, *p_buffer_size, dummy, dummy, nnzb, 0, 32, stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int>(
+                handle, nnzb, startbit, endbit, p_buffer_size)));
 
         *p_buffer_size = ((*p_buffer_size - 1) / 256 + 1) * 256;
 
@@ -397,6 +390,7 @@ namespace rocsparse
                                      size_t*              p_buffer_size)                       \
     try                                                                                        \
     {                                                                                          \
+        ROCSPARSE_ROUTINE_TRACE;                                                               \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsc_buffer_size_template(handle,          \
                                                                               mb,              \
                                                                               nb,              \
@@ -442,6 +436,7 @@ C_IMPL(rocsparse_zgebsr2gebsc_buffer_size, rocsparse_double_complex);
                                      void*                buffer)                \
     try                                                                          \
     {                                                                            \
+        ROCSPARSE_ROUTINE_TRACE;                                                 \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::gebsr2gebsc_template(handle,        \
                                                                   mb,            \
                                                                   nb,            \
