@@ -159,6 +159,11 @@ namespace rocsparse
             rocsparse::internal_dnvec_print<_Float16>(out, nmemb, hind);
             break;
         }
+        case rocsparse_datatype_bf16_r:
+        {
+            rocsparse::internal_dnvec_print<rocsparse_bfloat16>(out, nmemb, hind);
+            break;
+        }
         case rocsparse_datatype_f32_r:
         {
             rocsparse::internal_dnvec_print<float>(out, nmemb, hind);
@@ -225,6 +230,11 @@ namespace rocsparse
             rocsparse::internal_dnmat_print<_Float16>(out, m, n, hind, m);
             break;
         }
+        case rocsparse_datatype_bf16_r:
+        {
+            rocsparse::internal_dnmat_print<rocsparse_bfloat16>(out, m, n, hind, m);
+            break;
+        }
         case rocsparse_datatype_f32_r:
         {
             rocsparse::internal_dnmat_print<float>(out, m, n, hind, m);
