@@ -1,5 +1,5 @@
 /* ************************************************************************
- * Copyright (C) 2023 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -21,7 +21,7 @@
  *
  * ************************************************************************ */
 
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 #include "rocsparse_gcoo2coo_aos.hpp"
 #include "rocsparse_gcoo_aos2coo.hpp"
@@ -78,6 +78,8 @@ typedef rocsparse_status (*convert_type)(rocsparse_handle
 //
 FUNCTION_CONVERT(coo, coo_aos)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -92,18 +94,20 @@ FUNCTION_CONVERT(coo, coo_aos)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo2coo_aos_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_coo2coo_aos_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo2coo_aos(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_coo2coo_aos(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -111,6 +115,8 @@ FUNCTION_CONVERT(coo, coo_aos)
 //
 FUNCTION_CONVERT(coo, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -125,18 +131,20 @@ FUNCTION_CONVERT(coo, csr)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo2csr_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_coo2csr_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo2csr(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_coo2csr(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -144,7 +152,9 @@ FUNCTION_CONVERT(coo, csr)
 //
 FUNCTION_CONVERT(coo, csc)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
@@ -154,7 +164,9 @@ FUNCTION_CONVERT(coo, csc)
 //
 FUNCTION_CONVERT(coo, ell)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
@@ -164,7 +176,9 @@ FUNCTION_CONVERT(coo, ell)
 //
 FUNCTION_CONVERT(coo, bsr)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
@@ -174,19 +188,21 @@ FUNCTION_CONVERT(coo, bsr)
 //
 FUNCTION_CONVERT(coo_aos, coo)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo_aos2coo_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_coo_aos2coo_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo_aos2coo(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_coo_aos2coo(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
 
@@ -201,7 +217,9 @@ FUNCTION_CONVERT(coo_aos, coo)
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -209,19 +227,21 @@ FUNCTION_CONVERT(coo_aos, coo)
 //
 FUNCTION_CONVERT(coo_aos, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo_aos2csr_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_coo_aos2csr_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_coo_aos2csr(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_coo_aos2csr(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
 
@@ -235,7 +255,9 @@ FUNCTION_CONVERT(coo_aos, csr)
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -243,19 +265,21 @@ FUNCTION_CONVERT(coo_aos, csr)
 //
 FUNCTION_CONVERT(csr, coo_aos)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2coo_aos_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_csr2coo_aos_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2coo_aos(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_csr2coo_aos(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
 
@@ -270,7 +294,9 @@ FUNCTION_CONVERT(csr, coo_aos)
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -278,19 +304,21 @@ FUNCTION_CONVERT(csr, coo_aos)
 //
 FUNCTION_CONVERT(csr, coo)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2coo_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_csr2coo_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2coo(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_csr2coo(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
 
@@ -305,7 +333,9 @@ FUNCTION_CONVERT(csr, coo)
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -313,6 +343,7 @@ FUNCTION_CONVERT(csr, coo)
 //
 FUNCTION_CONVERT(csr, csc)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     switch(stage_)
     {
@@ -328,18 +359,20 @@ FUNCTION_CONVERT(csr, csc)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2csc_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_csr2csc_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2csc(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_csr2csc(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -347,6 +380,8 @@ FUNCTION_CONVERT(csr, csc)
 //
 FUNCTION_CONVERT(csr, ell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -358,7 +393,7 @@ FUNCTION_CONVERT(csr, ell)
     {
         int64_t ell_width;
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2ell_width(handle, source_, target_, &ell_width));
+            rocsparse::spmat_csr2ell_width(handle, source_, target_, &ell_width));
 
         //
         // Update target.
@@ -371,19 +406,21 @@ FUNCTION_CONVERT(csr, ell)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2ell_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_csr2ell_buffer_size(handle, source_, target_, buffer_size_));
 
         return rocsparse_status_success;
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2ell(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_csr2ell(handle, source_, target_, buffer_size_[0], buffer_));
 
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -391,6 +428,8 @@ FUNCTION_CONVERT(csr, ell)
 //
 FUNCTION_CONVERT(csr, bsr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -400,7 +439,7 @@ FUNCTION_CONVERT(csr, bsr)
     case _rocsparse_sparse_to_sparse_descr::stage_analysis:
     {
         int64_t nnzb;
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_spmat_csr2bsr_nnz(handle, source_, target_, &nnzb));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::spmat_csr2bsr_nnz(handle, source_, target_, &nnzb));
         target_->nnz = nnzb;
         return rocsparse_status_success;
     }
@@ -408,18 +447,20 @@ FUNCTION_CONVERT(csr, bsr)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2bsr_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_csr2bsr_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csr2bsr(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_csr2bsr(handle, source_, target_, buffer_size_[0], buffer_));
 
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -427,6 +468,8 @@ FUNCTION_CONVERT(csr, bsr)
 //
 FUNCTION_CONVERT(csc, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -442,19 +485,21 @@ FUNCTION_CONVERT(csc, csr)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csc2csr_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_csc2csr_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
 
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_csc2csr(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_csc2csr(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -462,6 +507,8 @@ FUNCTION_CONVERT(csc, csr)
 //
 FUNCTION_CONVERT(ell, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -471,7 +518,7 @@ FUNCTION_CONVERT(ell, csr)
     case _rocsparse_sparse_to_sparse_descr::stage_analysis:
     {
         int64_t csr_nnz;
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_spmat_ell2csr_nnz(handle, source_, target_, &csr_nnz));
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::spmat_ell2csr_nnz(handle, source_, target_, &csr_nnz));
         target_->nnz = csr_nnz;
         return rocsparse_status_success;
     }
@@ -479,18 +526,20 @@ FUNCTION_CONVERT(ell, csr)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_ell2csr_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_ell2csr_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_ell2csr(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_ell2csr(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -498,6 +547,8 @@ FUNCTION_CONVERT(ell, csr)
 //
 FUNCTION_CONVERT(bsr, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_analysis:
@@ -512,18 +563,20 @@ FUNCTION_CONVERT(bsr, csr)
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_bsr2csr_buffer_size(handle, source_, target_, buffer_size_));
+            rocsparse::spmat_bsr2csr_buffer_size(handle, source_, target_, buffer_size_));
         return rocsparse_status_success;
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
         RETURN_IF_ROCSPARSE_ERROR(
-            rocsparse_spmat_bsr2csr(handle, source_, target_, buffer_size_[0], buffer_));
+            rocsparse::spmat_bsr2csr(handle, source_, target_, buffer_size_[0], buffer_));
         return rocsparse_status_success;
     }
     }
 
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 //
@@ -531,7 +584,9 @@ FUNCTION_CONVERT(bsr, csr)
 //
 FUNCTION_CONVERT(csc, coo_aos)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
@@ -541,87 +596,114 @@ FUNCTION_CONVERT(csc, coo_aos)
 //
 FUNCTION_CONVERT(coo_aos, ell)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 
 FUNCTION_CONVERT(coo_aos, bsr)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 
 FUNCTION_CONVERT(coo_aos, csc)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(csc, coo)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(csc, ell)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(csc, bsr)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(ell, coo)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(ell, coo_aos)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(ell, csc)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(ell, bsr)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(bsr, coo)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(bsr, coo_aos)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(bsr, csc)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
 FUNCTION_CONVERT(bsr, ell)
 {
-    RETURN_IF_ROCSPARSE_ERROR(rocsparse_sparse_to_csr_to_sparse(
+    ROCSPARSE_ROUTINE_TRACE;
+
+    RETURN_IF_ROCSPARSE_ERROR(rocsparse::sparse_to_csr_to_sparse(
         handle, descr_, source_, target_, stage_, buffer_size_, buffer_));
     return rocsparse_status_success;
 }
@@ -631,6 +713,8 @@ FUNCTION_CONVERT(bsr, ell)
 //
 FUNCTION_CONVERT(coo, coo)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
@@ -648,15 +732,20 @@ FUNCTION_CONVERT(coo, coo)
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_internal_spmat_transfer_from(handle, target_, source_));
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::internal_spmat_transfer_from(handle, target_, source_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 FUNCTION_CONVERT(coo_aos, coo_aos)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
@@ -674,15 +763,20 @@ FUNCTION_CONVERT(coo_aos, coo_aos)
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_internal_spmat_transfer_from(handle, target_, source_));
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::internal_spmat_transfer_from(handle, target_, source_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 FUNCTION_CONVERT(csr, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
@@ -701,15 +795,20 @@ FUNCTION_CONVERT(csr, csr)
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_internal_spmat_transfer_from(handle, target_, source_));
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::internal_spmat_transfer_from(handle, target_, source_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 FUNCTION_CONVERT(csc, csc)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
@@ -727,15 +826,20 @@ FUNCTION_CONVERT(csc, csc)
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_internal_spmat_transfer_from(handle, target_, source_));
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::internal_spmat_transfer_from(handle, target_, source_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 FUNCTION_CONVERT(ell, ell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
@@ -757,15 +861,20 @@ FUNCTION_CONVERT(ell, ell)
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_internal_spmat_transfer_from(handle, target_, source_));
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::internal_spmat_transfer_from(handle, target_, source_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 FUNCTION_CONVERT(bsr, bsr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     switch(stage_)
     {
     case _rocsparse_sparse_to_sparse_descr::stage_buffer_size_compute:
@@ -783,15 +892,20 @@ FUNCTION_CONVERT(bsr, bsr)
     }
     case _rocsparse_sparse_to_sparse_descr::stage_compute:
     {
-        RETURN_IF_ROCSPARSE_ERROR(rocsparse_internal_spmat_transfer_from(handle, target_, source_));
+        RETURN_IF_ROCSPARSE_ERROR(
+            rocsparse::internal_spmat_transfer_from(handle, target_, source_));
         return rocsparse_status_success;
     }
     }
+    // LCOV_EXCL_START
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+    // LCOV_EXCL_STOP
 }
 
 FUNCTION_CONVERT(bell, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 
@@ -800,51 +914,75 @@ FUNCTION_CONVERT(bell, bell)
 //
 FUNCTION_CONVERT(coo, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(coo_aos, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(csr, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(csc, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(ell, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(bsr, bell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 
 FUNCTION_CONVERT(bell, coo)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(bell, coo_aos)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(bell, csr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(bell, csc)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(bell, ell)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 FUNCTION_CONVERT(bell, bsr)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
 }
 
@@ -910,6 +1048,7 @@ static convert_type s_conversion_table[7][7]{
 rocsparse_status rocsparse_mat_descr_are_same(const rocsparse_mat_descr source,
                                               const rocsparse_mat_descr target)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     RETURN_ROCSPARSE_ERROR_IF(rocsparse_status_type_mismatch, source->type != target->type);
 
@@ -929,15 +1068,17 @@ rocsparse_status rocsparse_mat_descr_are_same(const rocsparse_mat_descr source,
     return rocsparse_status_success;
 }
 
-rocsparse_status rocsparse_internal_sparse_to_sparse(rocsparse_handle                 handle,
-                                                     rocsparse_sparse_to_sparse_descr descr,
-                                                     rocsparse_const_spmat_descr      source,
-                                                     rocsparse_spmat_descr            target,
-                                                     rocsparse_sparse_to_sparse_stage stage,
-                                                     size_t*                          buffer_size,
-                                                     void*                            buffer,
-                                                     bool compute_buffer_size)
+rocsparse_status rocsparse::internal_sparse_to_sparse(rocsparse_handle                 handle,
+                                                      rocsparse_sparse_to_sparse_descr descr,
+                                                      rocsparse_const_spmat_descr      source,
+                                                      rocsparse_spmat_descr            target,
+                                                      rocsparse_sparse_to_sparse_stage stage,
+                                                      size_t*                          buffer_size,
+                                                      void*                            buffer,
+                                                      bool compute_buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     //
     // Batched not yet supported.
     //
