@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,13 +24,13 @@
 
 #include "rocsparse_csrgemm_multadd.hpp"
 #include "../conversion/rocsparse_identity.hpp"
-#include "control.h"
 #include "csrgemm_device.h"
 #include "internal/extra/rocsparse_csrgemm.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csrgemm_calc.hpp"
 #include "rocsparse_csrgemm_mult.hpp"
 #include "rocsparse_csrgemm_scal.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 rocsparse_status rocsparse::csrgemm_multadd_quickreturn(rocsparse_handle          handle,
                                                         rocsparse_operation       trans_A,
@@ -62,6 +62,7 @@ rocsparse_status rocsparse::csrgemm_multadd_quickreturn(rocsparse_handle
                                                         const rocsparse_mat_info  info_C,
                                                         void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     if(m == 0 || n == 0)
     {
@@ -127,6 +128,8 @@ rocsparse_status rocsparse::csrgemm_multadd_core(rocsparse_handle          handl
                                                  const rocsparse_mat_info  info_C,
                                                  void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const bool mul = info_C->csrgemm_info->mul;
     const bool add = info_C->csrgemm_info->add;
     if(mul == true && add == true)
@@ -138,72 +141,36 @@ rocsparse_status rocsparse::csrgemm_multadd_core(rocsparse_handle          handl
                 "((trans_A != rocsparse_operation_none) || (trans_B != rocsparse_operation_none))");
         }
         // Perform gemm calculation
-        if(handle->pointer_mode == rocsparse_pointer_mode_device)
-        {
-            RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_calc_template(handle,
-                                                                       trans_A,
-                                                                       trans_B,
-                                                                       m,
-                                                                       n,
-                                                                       k,
-                                                                       alpha,
-                                                                       descr_A,
-                                                                       nnz_A,
-                                                                       csr_val_A,
-                                                                       csr_row_ptr_A,
-                                                                       csr_col_ind_A,
-                                                                       descr_B,
-                                                                       nnz_B,
-                                                                       csr_val_B,
-                                                                       csr_row_ptr_B,
-                                                                       csr_col_ind_B,
-                                                                       beta,
-                                                                       descr_D,
-                                                                       nnz_D,
-                                                                       csr_val_D,
-                                                                       csr_row_ptr_D,
-                                                                       csr_col_ind_D,
-                                                                       descr_C,
-                                                                       csr_val_C,
-                                                                       csr_row_ptr_C,
-                                                                       csr_col_ind_C,
-                                                                       info_C,
-                                                                       temp_buffer));
-            return rocsparse_status_success;
-        }
-        else
-        {
-            RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_calc_template(handle,
-                                                                       trans_A,
-                                                                       trans_B,
-                                                                       m,
-                                                                       n,
-                                                                       k,
-                                                                       *alpha,
-                                                                       descr_A,
-                                                                       nnz_A,
-                                                                       csr_val_A,
-                                                                       csr_row_ptr_A,
-                                                                       csr_col_ind_A,
-                                                                       descr_B,
-                                                                       nnz_B,
-                                                                       csr_val_B,
-                                                                       csr_row_ptr_B,
-                                                                       csr_col_ind_B,
-                                                                       *beta,
-                                                                       descr_D,
-                                                                       nnz_D,
-                                                                       csr_val_D,
-                                                                       csr_row_ptr_D,
-                                                                       csr_col_ind_D,
-                                                                       descr_C,
-                                                                       csr_val_C,
-                                                                       csr_row_ptr_C,
-                                                                       csr_col_ind_C,
-                                                                       info_C,
-                                                                       temp_buffer));
-            return rocsparse_status_success;
-        }
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_calc_template(handle,
+                                                                   trans_A,
+                                                                   trans_B,
+                                                                   m,
+                                                                   n,
+                                                                   k,
+                                                                   alpha,
+                                                                   descr_A,
+                                                                   nnz_A,
+                                                                   csr_val_A,
+                                                                   csr_row_ptr_A,
+                                                                   csr_col_ind_A,
+                                                                   descr_B,
+                                                                   nnz_B,
+                                                                   csr_val_B,
+                                                                   csr_row_ptr_B,
+                                                                   csr_col_ind_B,
+                                                                   beta,
+                                                                   descr_D,
+                                                                   nnz_D,
+                                                                   csr_val_D,
+                                                                   csr_row_ptr_D,
+                                                                   csr_col_ind_D,
+                                                                   descr_C,
+                                                                   csr_val_C,
+                                                                   csr_row_ptr_C,
+                                                                   csr_col_ind_C,
+                                                                   info_C,
+                                                                   temp_buffer));
+        return rocsparse_status_success;
     }
     else
     {
