@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2019-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2019-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -29,7 +29,7 @@
 #include "rocsparse_csrgemm_numeric_mult.hpp"
 #include "rocsparse_csrgemm_numeric_multadd.hpp"
 #include "rocsparse_csrgemm_numeric_scal.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 template <typename I, typename J, typename T>
 rocsparse_status rocsparse::csrgemm_numeric_core(rocsparse_handle          handle,
@@ -63,6 +63,8 @@ rocsparse_status rocsparse::csrgemm_numeric_core(rocsparse_handle          handl
                                                  const rocsparse_mat_info  info_C,
                                                  void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const bool mul = info_C->csrgemm_info->mul;
     const bool add = info_C->csrgemm_info->add;
     if(mul && add)
@@ -193,6 +195,8 @@ rocsparse_status rocsparse::csrgemm_numeric_quickreturn(rocsparse_handle
                                                         const rocsparse_mat_info  info_C,
                                                         void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const bool mul = info_C->csrgemm_info->mul;
     const bool add = info_C->csrgemm_info->add;
 
@@ -340,6 +344,8 @@ namespace rocsparse
                                                      const rocsparse_mat_info  info_C, //28
                                                      void*                     temp_buffer) //29
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_ENUM(1, trans_A);
         ROCSPARSE_CHECKARG_ENUM(2, trans_B);
@@ -482,6 +488,8 @@ namespace rocsparse
     template <typename... P>
     static rocsparse_status csrgemm_numeric_impl(P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         rocsparse::log_trace("rocsparse_csrgemm_numeric", p...);
 
         const rocsparse_status status = rocsparse::csrgemm_numeric_checkarg(p...);
@@ -582,6 +590,7 @@ INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
                                      void*                     temp_buffer)      \
     try                                                                          \
     {                                                                            \
+        ROCSPARSE_ROUTINE_TRACE;                                                 \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_numeric_impl(handle,        \
                                                                   trans_A,       \
                                                                   trans_B,       \
