@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -26,11 +26,11 @@
 #include "internal/extra/rocsparse_csrgemm.h"
 #include "rocsparse_csrgemm.hpp"
 
-#include "common.h"
-#include "control.h"
+#include "rocsparse_common.hpp"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csrgemm_numeric_calc.hpp"
 #include "rocsparse_csrgemm_numeric_multadd.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 rocsparse_status rocsparse::csrgemm_numeric_multadd_quickreturn(rocsparse_handle    handle,
                                                                 rocsparse_operation trans_A,
@@ -63,6 +63,8 @@ rocsparse_status rocsparse::csrgemm_numeric_multadd_quickreturn(rocsparse_handle
                                                                 const rocsparse_mat_info info_C,
                                                                 void* temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const bool mul = info_C->csrgemm_info->mul;
     const bool add = info_C->csrgemm_info->add;
     if(mul == true && add == true)
@@ -115,13 +117,15 @@ inline rocsparse_status rocsparse::csrgemm_numeric_multadd_core(rocsparse_handle
                                                                 const rocsparse_mat_info info_C,
                                                                 void* temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const bool mul = info_C->csrgemm_info->mul;
     const bool add = info_C->csrgemm_info->add;
     if(mul == true && add == true)
     {
         if((k == 0 || nnz_A == 0 || nnz_B == 0) && (nnz_D == 0))
         {
-            ROCSPARSE_RETURN_STATUS(success);
+            return rocsparse_status_success;
         }
 
         if(descr_A->type != rocsparse_matrix_type_general)
@@ -168,23 +172,24 @@ inline rocsparse_status rocsparse::csrgemm_numeric_multadd_core(rocsparse_handle
 
         if(descr_D->type != rocsparse_matrix_type_general)
         {
-            ROCSPARSE_RETURN_STATUS(not_implemented);
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
         }
 
         if(((trans_A != rocsparse_operation_none) || (trans_B != rocsparse_operation_none)))
         {
-            ROCSPARSE_RETURN_STATUS(not_implemented);
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
         }
 
         if(descr_A->type != rocsparse_matrix_type_general)
         {
-            ROCSPARSE_RETURN_STATUS(not_implemented);
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
         }
 
         if(descr_B->type != rocsparse_matrix_type_general)
         {
-            ROCSPARSE_RETURN_STATUS(not_implemented);
+            RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_not_implemented);
         }
+
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::csrgemm_numeric_calc_template(handle,
                                                                            trans_A,
                                                                            trans_B,
