@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2023-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2023-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,11 +22,12 @@
  *
  * ************************************************************************ */
 
-#include "control.h"
 #include "csrgemm_device.h"
+#include "rocsparse_common.h"
+#include "rocsparse_control.hpp"
 #include "rocsparse_csrgemm_nnz_calc.hpp"
 #include "rocsparse_csrgemm_scal.hpp"
-#include "utility.h"
+#include "rocsparse_utility.hpp"
 
 template <typename I>
 rocsparse_status rocsparse::csrgemm_scal_nnz_quickreturn(rocsparse_handle          handle,
@@ -42,6 +43,7 @@ rocsparse_status rocsparse::csrgemm_scal_nnz_quickreturn(rocsparse_handle
                                                          const rocsparse_mat_info  info_C,
                                                          void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     // Quick return if possible
     if(m == 0 || n == 0)
@@ -57,14 +59,8 @@ rocsparse_status rocsparse::csrgemm_scal_nnz_quickreturn(rocsparse_handle
 
         if(csr_row_ptr_C != nullptr)
         {
-            RETURN_IF_HIPLAUNCHKERNELGGL_ERROR((rocsparse::set_array_to_value<1024>),
-                                               dim3(m / 1024 + 1),
-                                               dim3(1024),
-                                               0,
-                                               handle->stream,
-                                               (m + 1),
-                                               csr_row_ptr_C,
-                                               static_cast<I>(descr_C->base));
+            RETURN_IF_ROCSPARSE_ERROR(
+                rocsparse::valset(handle, m + 1, static_cast<I>(descr_C->base), csr_row_ptr_C));
         }
         return rocsparse_status_success;
     }
@@ -105,6 +101,8 @@ rocsparse_status rocsparse::csrgemm_scal_nnz_core(rocsparse_handle          hand
                                                   const rocsparse_mat_info  info_C,
                                                   void*                     temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     const bool mul = info_C->csrgemm_info->mul;
     const bool add = info_C->csrgemm_info->add;
     if(mul == false && add == true)
