@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2018-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2018-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,31 +24,10 @@
 
 #pragma once
 
-#include "common.h"
+#include "rocsparse_common.hpp"
 
 namespace rocsparse
 {
-    // Scale kernel for beta != 1.0
-    template <uint32_t BLOCKSIZE, typename I, typename Y, typename T>
-    ROCSPARSE_DEVICE_ILF void coomv_scale_device(I size, T beta, Y* __restrict__ data)
-    {
-        const I gid = hipBlockIdx_x * BLOCKSIZE + hipThreadIdx_x;
-
-        if(gid >= size)
-        {
-            return;
-        }
-
-        if(beta == 0)
-        {
-            data[gid] = static_cast<Y>(0);
-        }
-        else
-        {
-            data[gid] = static_cast<Y>(beta * data[gid]);
-        }
-    }
-
     // Implementation motivated by papers 'Efficient Sparse Matrix-Vector Multiplication on CUDA',
     // 'Implementing Sparse Matrix-Vector Multiplication on Throughput-Oriented Processors' and
     // 'Segmented operations for sparse matrix computation on vector multiprocessors'
