@@ -296,7 +296,7 @@ namespace rocsparse
         int wid = hipThreadIdx_x / WF_SIZE;
         // Scalarize wid, i.e. move it from a vector register to a scalar register, so all dependent
         // values can be loaded or computed with scalar instructions (idx, row, row_begin...)
-        wid = __builtin_amdgcn_readfirstlane(wid);
+        wid = rocsparse::read_first_lane(wid);
 
         // Index into the row map
         J idx = hipBlockIdx_x * (BLOCKSIZE / WF_SIZE) + wid;
