@@ -93,7 +93,7 @@ namespace rocsparse
             float f1 = fk;
 #pragma unroll 7
             for(int s = 1; s < 8; ++s)
-                f1 = f1 + __shfl(fk, local_i + s * block_size, 64);
+                f1 = f1 + rocsparse::shfl(fk, local_i + s * block_size, 64);
 
             f1 = beta * y[global_row] + alpha * f1;
             if(local_j == 0)
@@ -561,6 +561,9 @@ INSTANTIATE_MIXED(float, int64_t, int64_t, int8_t, int8_t, float);
 INSTANTIATE_MIXED(float, int32_t, int32_t, _Float16, _Float16, float);
 INSTANTIATE_MIXED(float, int64_t, int32_t, _Float16, _Float16, float);
 INSTANTIATE_MIXED(float, int64_t, int64_t, _Float16, _Float16, float);
+INSTANTIATE_MIXED(float, int32_t, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE_MIXED(float, int64_t, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE_MIXED(float, int64_t, int64_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
 INSTANTIATE_MIXED(rocsparse_float_complex,
                   int32_t,
                   int32_t,
