@@ -781,6 +781,8 @@ INSTANTIATE_MIXED_ANALYSIS(int32_t, int32_t);
 INSTANTIATE_MIXED_ANALYSIS(int64_t, int32_t);
 INSTANTIATE_MIXED_ANALYSIS(int32_t, _Float16);
 INSTANTIATE_MIXED_ANALYSIS(int64_t, _Float16);
+INSTANTIATE_MIXED_ANALYSIS(int32_t, rocsparse_bfloat16);
+INSTANTIATE_MIXED_ANALYSIS(int64_t, rocsparse_bfloat16);
 #undef INSTANTIATE_MIXED_ANALYSIS
 
 #define INSTANTIATE_MIXED(T, I, A, X, Y)                                                          \
@@ -806,6 +808,8 @@ INSTANTIATE_MIXED(float, int32_t, int8_t, int8_t, float);
 INSTANTIATE_MIXED(float, int64_t, int8_t, int8_t, float);
 INSTANTIATE_MIXED(float, int32_t, _Float16, _Float16, float);
 INSTANTIATE_MIXED(float, int64_t, _Float16, _Float16, float);
+INSTANTIATE_MIXED(float, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE_MIXED(float, int64_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
 INSTANTIATE_MIXED(
     rocsparse_float_complex, int32_t, float, rocsparse_float_complex, rocsparse_float_complex);
 INSTANTIATE_MIXED(
