@@ -710,6 +710,9 @@ INSTANTIATE(int64_t, int64_t, int8_t);
 INSTANTIATE(int32_t, int32_t, _Float16);
 INSTANTIATE(int64_t, int32_t, _Float16);
 INSTANTIATE(int64_t, int64_t, _Float16);
+INSTANTIATE(int32_t, int32_t, rocsparse_bfloat16);
+INSTANTIATE(int64_t, int32_t, rocsparse_bfloat16);
+INSTANTIATE(int64_t, int64_t, rocsparse_bfloat16);
 
 #undef INSTANTIATE
 
@@ -785,6 +788,9 @@ INSTANTIATE(float, int64_t, int64_t, int8_t, int8_t, float);
 INSTANTIATE(float, int32_t, int32_t, _Float16, _Float16, float);
 INSTANTIATE(float, int64_t, int32_t, _Float16, _Float16, float);
 INSTANTIATE(float, int64_t, int64_t, _Float16, _Float16, float);
+INSTANTIATE(float, int32_t, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE(float, int64_t, int32_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
+INSTANTIATE(float, int64_t, int64_t, rocsparse_bfloat16, rocsparse_bfloat16, float);
 INSTANTIATE(rocsparse_float_complex,
             int32_t,
             int32_t,
