@@ -775,14 +775,22 @@ INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
 INSTANTIATE(int32_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int32_t, int32_t, rocsparse_double_complex);
+
 INSTANTIATE(int64_t, int32_t, float);
 INSTANTIATE(int64_t, int32_t, double);
 INSTANTIATE(int64_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int32_t, rocsparse_double_complex);
+
 INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
+
+INSTANTIATE(int32_t, int64_t, float);
+INSTANTIATE(int32_t, int64_t, double);
+INSTANTIATE(int32_t, int64_t, rocsparse_float_complex);
+INSTANTIATE(int32_t, int64_t, rocsparse_double_complex);
+
 #undef INSTANTIATE
 
 #define INSTANTIATE(ITYPE, JTYPE, TTYPE)                          \
@@ -804,14 +812,22 @@ INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
 INSTANTIATE(int32_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int32_t, int32_t, rocsparse_double_complex);
+
 INSTANTIATE(int64_t, int32_t, float);
 INSTANTIATE(int64_t, int32_t, double);
 INSTANTIATE(int64_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int32_t, rocsparse_double_complex);
+
 INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
+
+INSTANTIATE(int32_t, int64_t, float);
+INSTANTIATE(int32_t, int64_t, double);
+INSTANTIATE(int32_t, int64_t, rocsparse_float_complex);
+INSTANTIATE(int32_t, int64_t, rocsparse_double_complex);
+
 #undef INSTANTIATE
 
 /*
