@@ -140,12 +140,12 @@ namespace rocsparse
 
             // Gather values
             RETURN_IF_ROCSPARSE_ERROR(
-                rocsparse::gthr_template(handle,
-                                         nnz,
-                                         csr_val,
-                                         csrt_val,
-                                         (const I*)csrsv->get_transposed_perm(),
-                                         rocsparse_index_base_zero));
+                (rocsparse::gthr_template<I, T>(handle,
+                                                nnz,
+                                                csr_val,
+                                                csrt_val,
+                                                (const I*)csrsv->get_transposed_perm(),
+                                                rocsparse_index_base_zero)));
 
             if(trans == rocsparse_operation_conjugate_transpose)
             {
@@ -379,14 +379,22 @@ INSTANTIATE(int32_t, int32_t, float);
 INSTANTIATE(int32_t, int32_t, double);
 INSTANTIATE(int32_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int32_t, int32_t, rocsparse_double_complex);
+
 INSTANTIATE(int64_t, int32_t, float);
 INSTANTIATE(int64_t, int32_t, double);
 INSTANTIATE(int64_t, int32_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int32_t, rocsparse_double_complex);
+
 INSTANTIATE(int64_t, int64_t, float);
 INSTANTIATE(int64_t, int64_t, double);
 INSTANTIATE(int64_t, int64_t, rocsparse_float_complex);
 INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
+
+INSTANTIATE(int32_t, int64_t, float);
+INSTANTIATE(int32_t, int64_t, double);
+INSTANTIATE(int32_t, int64_t, rocsparse_float_complex);
+INSTANTIATE(int32_t, int64_t, rocsparse_double_complex);
+
 #undef INSTANTIATE
 
 /*
