@@ -51,6 +51,7 @@ const char* rocsparse::enum_utils::to_string(rocsparse_spmv_alg value_)
         CASE(rocsparse_spmv_alg_coo_atomic);
         CASE(rocsparse_spmv_alg_bsr);
         CASE(rocsparse_spmv_alg_csr_lrb);
+        CASE(rocsparse_spmv_alg_csr_nnzsplit);
 #undef CASE
     }
     // LCOV_EXCL_START
@@ -104,6 +105,7 @@ rocsparse_status rocsparse::check_spmv_alg(rocsparse_format format, rocsparse_sp
         case rocsparse_spmv_alg_csr_rowsplit:
         case rocsparse_spmv_alg_csr_adaptive:
         case rocsparse_spmv_alg_csr_lrb:
+        case rocsparse_spmv_alg_csr_nnzsplit:
         {
             return rocsparse_status_success;
         }
@@ -136,6 +138,7 @@ rocsparse_status rocsparse::check_spmv_alg(rocsparse_format format, rocsparse_sp
         case rocsparse_spmv_alg_bsr:
         case rocsparse_spmv_alg_ell:
         case rocsparse_spmv_alg_csr_lrb:
+        case rocsparse_spmv_alg_csr_nnzsplit:
         {
             // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
@@ -160,6 +163,7 @@ rocsparse_status rocsparse::check_spmv_alg(rocsparse_format format, rocsparse_sp
         case rocsparse_spmv_alg_coo:
         case rocsparse_spmv_alg_coo_atomic:
         case rocsparse_spmv_alg_csr_lrb:
+        case rocsparse_spmv_alg_csr_nnzsplit:
         {
             // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
@@ -181,6 +185,7 @@ rocsparse_status rocsparse::check_spmv_alg(rocsparse_format format, rocsparse_sp
         case rocsparse_spmv_alg_bsr:
         case rocsparse_spmv_alg_coo_atomic:
         case rocsparse_spmv_alg_csr_lrb:
+        case rocsparse_spmv_alg_csr_nnzsplit:
         {
             // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
@@ -206,6 +211,7 @@ rocsparse_status rocsparse::check_spmv_alg(rocsparse_format format, rocsparse_sp
         case rocsparse_spmv_alg_coo:
         case rocsparse_spmv_alg_coo_atomic:
         case rocsparse_spmv_alg_csr_lrb:
+        case rocsparse_spmv_alg_csr_nnzsplit:
         {
             // LCOV_EXCL_START
             RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
@@ -244,6 +250,12 @@ rocsparse_status rocsparse::spmv_alg2csrmv_alg(rocsparse_spmv_alg    spmv_alg,
         return rocsparse_status_success;
     }
 
+    case rocsparse_spmv_alg_csr_nnzsplit:
+    {
+        target = rocsparse::csrmv_alg_nnzsplit;
+        return rocsparse_status_success;
+    }
+
     case rocsparse_spmv_alg_coo:
     case rocsparse_spmv_alg_coo_atomic:
     case rocsparse_spmv_alg_bsr:
@@ -285,6 +297,7 @@ rocsparse_status rocsparse::spmv_alg2coomv_alg(rocsparse_spmv_alg   spmv_alg,
     case rocsparse_spmv_alg_bsr:
     case rocsparse_spmv_alg_ell:
     case rocsparse_spmv_alg_csr_lrb:
+    case rocsparse_spmv_alg_csr_nnzsplit:
     {
         // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
@@ -322,6 +335,7 @@ rocsparse_status rocsparse::spmv_alg2coomv_aos_alg(rocsparse_spmv_alg       spmv
     case rocsparse_spmv_alg_bsr:
     case rocsparse_spmv_alg_ell:
     case rocsparse_spmv_alg_csr_lrb:
+    case rocsparse_spmv_alg_csr_nnzsplit:
     {
         // LCOV_EXCL_START
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
