@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2020-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2020-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -24,30 +24,10 @@
 
 #pragma once
 
-#include "common.h"
+#include "rocsparse_common.hpp"
 
 namespace rocsparse
 {
-    template <uint32_t BLOCKSIZE, typename T>
-    ROCSPARSE_DEVICE_ILF void gemmi_scale_device(rocsparse_int size, T alpha, T* __restrict__ data)
-    {
-        rocsparse_int idx = hipBlockIdx_x * BLOCKSIZE + hipThreadIdx_x;
-
-        if(idx >= size)
-        {
-            return;
-        }
-
-        if(alpha == static_cast<T>(0))
-        {
-            data[idx] = static_cast<T>(0);
-        }
-        else
-        {
-            data[idx] *= alpha;
-        }
-    }
-
     template <uint32_t BLOCKSIZE, typename T>
     ROCSPARSE_DEVICE_ILF void gemmit_device(rocsparse_int m,
                                             rocsparse_int n,
