@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2021-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2021-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -22,12 +22,12 @@
  *
  * ************************************************************************ */
 
-#include "control.h"
 #include "internal/level3/rocsparse_bsrsm.h"
 #include "rocsparse_bsrsm.hpp"
-#include "utility.h"
+#include "rocsparse_control.hpp"
+#include "rocsparse_utility.hpp"
 
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 rocsparse_status rocsparse::bsrsm_buffer_size_quickreturn(rocsparse_handle          handle,
                                                           rocsparse_direction       dir,
@@ -44,6 +44,7 @@ rocsparse_status rocsparse::bsrsm_buffer_size_quickreturn(rocsparse_handle
                                                           rocsparse_mat_info        info,
                                                           size_t*                   buffer_size)
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     if(mb == 0 || nrhs == 0)
     {
@@ -71,6 +72,8 @@ namespace rocsparse
                                                        rocsparse_mat_info        info, //12
                                                        size_t*                   buffer_size) //13
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         ROCSPARSE_CHECKARG_HANDLE(0, handle);
         ROCSPARSE_CHECKARG_ENUM(1, dir);
         ROCSPARSE_CHECKARG_ENUM(2, trans_A);
@@ -136,8 +139,7 @@ rocsparse_status rocsparse::bsrsm_buffer_size_core(rocsparse_handle          han
                                                    rocsparse_mat_info        info,
                                                    size_t*                   buffer_size)
 {
-    // Stream
-    hipStream_t stream = handle->stream;
+    ROCSPARSE_ROUTINE_TRACE;
 
     // rocsparse_int max_nnz
     *buffer_size = 256;
@@ -156,15 +158,13 @@ rocsparse_status rocsparse::bsrsm_buffer_size_core(rocsparse_handle          han
     // int workspace2
     *buffer_size += ((sizeof(int) * mb - 1) / 256 + 1) * 256;
 
-    size_t         rocprim_size;
-    rocsparse_int* ptr  = reinterpret_cast<rocsparse_int*>(buffer_size);
-    int*           ptr2 = reinterpret_cast<int*>(buffer_size);
+    uint32_t startbit = 0;
+    uint32_t endbit   = rocsparse::clz(mb);
 
-    rocprim::double_buffer<rocsparse_int> dummy(ptr, ptr);
-    rocprim::double_buffer<int>           dummy2(ptr2, ptr2);
-
-    RETURN_IF_HIP_ERROR(
-        rocprim::radix_sort_pairs(nullptr, rocprim_size, dummy2, dummy, mb, 0, 32, stream));
+    size_t rocprim_size;
+    RETURN_IF_ROCSPARSE_ERROR(
+        (rocsparse::primitives::radix_sort_pairs_buffer_size<int, rocsparse_int>(
+            handle, mb, startbit, endbit, &rocprim_size)));
 
     // rocprim buffer
     *buffer_size += rocprim_size;
@@ -181,8 +181,9 @@ rocsparse_status rocsparse::bsrsm_buffer_size_core(rocsparse_handle          han
         size_t transpose_size;
 
         // Determine rocprim buffer size
-        RETURN_IF_HIP_ERROR(
-            rocprim::radix_sort_pairs(nullptr, transpose_size, dummy, dummy, nnzb, 0, 32, stream));
+        RETURN_IF_ROCSPARSE_ERROR(
+            (rocsparse::primitives::radix_sort_pairs_buffer_size<rocsparse_int, rocsparse_int>(
+                handle, nnzb, startbit, endbit, &transpose_size)));
 
         // rocPRIM does not support in-place sorting, so we need an additional buffer
         transpose_size += ((sizeof(rocsparse_int) * nnzb - 1) / 256 + 1) * 256;
@@ -199,6 +200,8 @@ namespace rocsparse
     template <typename... P>
     static rocsparse_status bsrsm_buffer_size_impl(P&&... p)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         rocsparse::log_trace("rocsparse_Xbsrsm_buffer_size", p...);
 
         const rocsparse_status status = rocsparse::bsrsm_buffer_size_checkarg(p...);
@@ -237,6 +240,7 @@ namespace rocsparse
                                      size_t*                   buffer_size)        \
     try                                                                            \
     {                                                                              \
+        ROCSPARSE_ROUTINE_TRACE;                                                   \
         RETURN_IF_ROCSPARSE_ERROR(rocsparse::bsrsm_buffer_size_impl(handle,        \
                                                                     dir,           \
                                                                     trans_A,       \
