@@ -40,29 +40,29 @@ namespace rocsparse
     rocsparse_status coomm_buffer_size_template(rocsparse_handle          handle,
                                                 rocsparse_operation       trans_A,
                                                 rocsparse_coomm_alg       alg,
-                                                I                         m,
-                                                I                         n,
-                                                I                         k,
+                                                int64_t                   m,
+                                                int64_t                   n,
+                                                int64_t                   k,
                                                 int64_t                   nnz,
-                                                I                         batch_count,
+                                                int64_t                   batch_count,
                                                 const rocsparse_mat_descr descr,
-                                                const A*                  coo_val,
-                                                const I*                  coo_row_ind,
-                                                const I*                  coo_col_ind,
+                                                const void*               coo_val,
+                                                const void*               coo_row_ind,
+                                                const void*               coo_col_ind,
                                                 size_t*                   buffer_size);
 
-    template <typename T, typename I, typename A>
+    template <typename I, typename A>
     rocsparse_status coomm_analysis_template(rocsparse_handle          handle,
                                              rocsparse_operation       trans_A,
                                              rocsparse_coomm_alg       alg,
-                                             I                         m,
-                                             I                         n,
-                                             I                         k,
+                                             int64_t                   m,
+                                             int64_t                   n,
+                                             int64_t                   k,
                                              int64_t                   nnz,
                                              const rocsparse_mat_descr descr,
-                                             const A*                  coo_val,
-                                             const I*                  coo_row_ind,
-                                             const I*                  coo_col_ind,
+                                             const void*               coo_val,
+                                             const void*               coo_row_ind,
+                                             const void*               coo_col_ind,
                                              void*                     temp_buffer);
 
     template <typename T, typename I, typename A, typename B, typename C>
@@ -99,27 +99,96 @@ namespace rocsparse
                                     rocsparse_operation       trans_A,
                                     rocsparse_operation       trans_B,
                                     rocsparse_coomm_alg       alg,
-                                    I                         m,
-                                    I                         n,
-                                    I                         k,
+                                    int64_t                   m,
+                                    int64_t                   n,
+                                    int64_t                   k,
                                     int64_t                   nnz,
-                                    I                         batch_count_A,
+                                    int64_t                   batch_count_A,
                                     int64_t                   batch_stride_A,
-                                    const T*                  alpha,
+                                    const void*               alpha,
                                     const rocsparse_mat_descr descr,
-                                    const A*                  coo_val,
-                                    const I*                  coo_row_ind,
-                                    const I*                  coo_col_ind,
-                                    const B*                  dense_B,
+                                    const void*               coo_val,
+                                    const void*               coo_row_ind,
+                                    const void*               coo_col_ind,
+                                    const void*               dense_B,
                                     int64_t                   ldb,
-                                    I                         batch_count_B,
+                                    int64_t                   batch_count_B,
                                     int64_t                   batch_stride_B,
                                     rocsparse_order           order_B,
-                                    const T*                  beta,
-                                    C*                        dense_C,
+                                    const void*               beta,
+                                    void*                     dense_C,
                                     int64_t                   ldc,
-                                    I                         batch_count_C,
+                                    int64_t                   batch_count_C,
                                     int64_t                   batch_stride_C,
                                     rocsparse_order           order_C,
                                     void*                     temp_buffer);
+
+    rocsparse_status coomm_buffer_size(rocsparse_handle          handle,
+                                       rocsparse_operation       trans_A,
+                                       rocsparse_coomm_alg       alg,
+                                       int64_t                   m,
+                                       int64_t                   n,
+                                       int64_t                   k,
+                                       int64_t                   nnz,
+                                       int64_t                   batch_count,
+                                       const rocsparse_mat_descr descr,
+                                       rocsparse_datatype        compute_datatype,
+                                       rocsparse_datatype        coo_val_datatype,
+                                       const void*               coo_val,
+                                       rocsparse_indextype       coo_row_ind_indextype,
+                                       const void*               coo_row_ind,
+                                       rocsparse_indextype       coo_col_ind_indextype,
+                                       const void*               coo_col_ind,
+                                       size_t*                   buffer_size);
+
+    rocsparse_status coomm_analysis(rocsparse_handle          handle,
+                                    rocsparse_operation       trans_A,
+                                    rocsparse_coomm_alg       alg,
+                                    int64_t                   m,
+                                    int64_t                   n,
+                                    int64_t                   k,
+                                    int64_t                   nnz,
+                                    const rocsparse_mat_descr descr,
+                                    rocsparse_datatype        coo_val_datatype,
+                                    const void*               coo_val,
+                                    rocsparse_indextype       coo_row_ind_indextype,
+                                    const void*               coo_row_ind,
+                                    rocsparse_indextype       coo_col_ind_indextype,
+                                    const void*               coo_col_ind,
+                                    void*                     temp_buffer);
+
+    rocsparse_status coomm(rocsparse_handle          handle,
+                           rocsparse_operation       trans_A,
+                           rocsparse_operation       trans_B,
+                           rocsparse_coomm_alg       alg,
+                           int64_t                   m,
+                           int64_t                   n,
+                           int64_t                   k,
+                           int64_t                   nnz,
+                           int64_t                   batch_count_A,
+                           int64_t                   batch_stride_A,
+                           rocsparse_datatype        alpha_datatype,
+                           const void*               alpha,
+                           const rocsparse_mat_descr descr,
+                           rocsparse_datatype        coo_val_datatype,
+                           const void*               coo_val,
+                           rocsparse_indextype       coo_row_ind_indextype,
+                           const void*               coo_row_ind,
+                           rocsparse_indextype       coo_col_ind_indextype,
+                           const void*               coo_col_ind,
+                           rocsparse_datatype        dense_B_datatype,
+                           const void*               dense_B,
+                           int64_t                   ldb,
+                           int64_t                   batch_count_B,
+                           int64_t                   batch_stride_B,
+                           rocsparse_order           order_B,
+                           rocsparse_datatype        beta_datatype,
+                           const void*               beta,
+                           rocsparse_datatype        dense_C_datatype,
+                           void*                     dense_C,
+                           int64_t                   ldc,
+                           int64_t                   batch_count_C,
+                           int64_t                   batch_stride_C,
+                           rocsparse_order           order_C,
+                           void*                     temp_buffer);
 }
