@@ -1,6 +1,6 @@
 /*! \file */
 /* ************************************************************************
- * Copyright (C) 2022-2024 Advanced Micro Devices, Inc. All rights Reserved.
+ * Copyright (C) 2022-2025 Advanced Micro Devices, Inc. All rights Reserved.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -23,10 +23,10 @@
  * ************************************************************************ */
 #include "internal/util/rocsparse_check_matrix_gebsr.h"
 #include "rocsparse_check_matrix_gebsr.hpp"
-#include "to_string.hpp"
-#include "utility.h"
+#include "rocsparse_enum_utils.hpp"
+#include "rocsparse_utility.hpp"
 
-#include <rocprim/rocprim.hpp>
+#include "rocsparse_primitives.hpp"
 
 #include "check_matrix_gebsr_device.h"
 
@@ -48,6 +48,8 @@ rocsparse_status rocsparse::check_matrix_gebsr_core(rocsparse_handle       handl
                                                     rocsparse_data_status* data_status,
                                                     void*                  temp_buffer)
 {
+    ROCSPARSE_ROUTINE_TRACE;
+
     *data_status = rocsparse_data_status_success;
 
     I start = 0;
@@ -60,8 +62,10 @@ rocsparse_status rocsparse::check_matrix_gebsr_core(rocsparse_handle       handl
 
     if(nnzb != (end - start))
     {
+        // LCOV_EXCL_START
         rocsparse::log_debug(handle, "GEBSR row pointer array does not match nnzb.");
         RETURN_IF_ROCSPARSE_ERROR(rocsparse_status_invalid_value);
+        // LCOV_EXCL_STOP
     }
 
     // Temporary buffer entry points
@@ -91,7 +95,7 @@ rocsparse_status rocsparse::check_matrix_gebsr_core(rocsparse_handle       handl
 
     if(*data_status != rocsparse_data_status_success)
     {
-        rocsparse::log_debug(handle, rocsparse::to_string(*data_status));
+        rocsparse::log_debug(handle, rocsparse::enum_utils::to_string(*data_status));
 
         return rocsparse_status_success;
     }
@@ -103,10 +107,6 @@ rocsparse_status rocsparse::check_matrix_gebsr_core(rocsparse_handle       handl
     // If columns are unsorted, then sort them in temp buffer
     if(storage == rocsparse_storage_mode_unsorted)
     {
-        uint32_t startbit = 0;
-        uint32_t endbit   = rocsparse::clz(nb);
-        size_t   size;
-
         // offsets buffer
         tmp_offsets = reinterpret_cast<I*>(ptr);
         ptr += ((sizeof(I) * mb) / 256 + 1) * 256;
@@ -128,99 +128,11 @@ rocsparse_status rocsparse::check_matrix_gebsr_core(rocsparse_handle       handl
                                            bsr_row_ptr,
                                            tmp_offsets);
 
-        RETURN_IF_HIP_ERROR(
-            hipMemcpyAsync(tmp_cols1, bsr_col_ind, sizeof(J) * nnzb, hipMemcpyDeviceToDevice));
-        RETURN_IF_HIP_ERROR(hipStreamSynchronize(handle->stream));
-
         // rocprim buffer
         void* tmp_rocprim = reinterpret_cast<void*>(ptr);
 
-        // Compute buffer size
-        rocprim::double_buffer<J> dummy(tmp_cols1, tmp_cols2);
-        RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys(nullptr,
-                                                               size,
-                                                               dummy,
-                                                               nnzb,
-                                                               mb,
-                                                               tmp_offsets,
-                                                               tmp_offsets + 1,
-                                                               startbit,
-                                                               endbit,
-                                                               handle->stream));
-
-        // Sort by keys
-        rocprim::double_buffer<J> keys(tmp_cols1, tmp_cols2);
-
-        // Determine blocksize and items per thread depending on average nnz per row
-        J avg_row_nnzb = nnzb / mb;
-
-        if(avg_row_nnzb < 64)
-        {
-            using config
-                = rocprim::segmented_radix_sort_config<6, 5, rocprim::kernel_config<64, 1>>;
-            RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys<config>(tmp_rocprim,
-                                                                           size,
-                                                                           keys,
-                                                                           nnzb,
-                                                                           mb,
-                                                                           tmp_offsets,
-                                                                           tmp_offsets + 1,
-                                                                           startbit,
-                                                                           endbit,
-                                                                           handle->stream));
-        }
-        else if(avg_row_nnzb < 128)
-        {
-            using config
-                = rocprim::segmented_radix_sort_config<6, 5, rocprim::kernel_config<64, 2>>;
-            RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys<config>(tmp_rocprim,
-                                                                           size,
-                                                                           keys,
-                                                                           nnzb,
-                                                                           mb,
-                                                                           tmp_offsets,
-                                                                           tmp_offsets + 1,
-                                                                           startbit,
-                                                                           endbit,
-                                                                           handle->stream));
-        }
-        else if(avg_row_nnzb < 256)
-        {
-            using config
-                = rocprim::segmented_radix_sort_config<6, 5, rocprim::kernel_config<64, 4>>;
-            RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys<config>(tmp_rocprim,
-                                                                           size,
-                                                                           keys,
-                                                                           nnzb,
-                                                                           mb,
-                                                                           tmp_offsets,
-                                                                           tmp_offsets + 1,
-                                                                           startbit,
-                                                                           endbit,
-                                                                           handle->stream));
-        }
-        else
-        {
-            RETURN_IF_HIP_ERROR(rocprim::segmented_radix_sort_keys(tmp_rocprim,
-                                                                   size,
-                                                                   keys,
-                                                                   nnzb,
-                                                                   mb,
-                                                                   tmp_offsets,
-                                                                   tmp_offsets + 1,
-                                                                   startbit,
-                                                                   endbit,
-                                                                   handle->stream));
-        }
-
-        if(keys.current() != tmp_cols2)
-        {
-            RETURN_IF_HIP_ERROR(hipMemcpyAsync(tmp_cols2,
-                                               keys.current(),
-                                               sizeof(J) * nnzb,
-                                               hipMemcpyDeviceToDevice,
-                                               handle->stream));
-        }
+        RETURN_IF_ROCSPARSE_ERROR(rocsparse::primitives::sort_csr_column_indices(
+            handle, mb, nb, nnzb, tmp_offsets, bsr_col_ind, tmp_cols1, tmp_cols2, tmp_rocprim));
     }
 
     const J* bsr_col_ind_sorted
@@ -256,7 +168,7 @@ rocsparse_status rocsparse::check_matrix_gebsr_core(rocsparse_handle       handl
 
     if(*data_status != rocsparse_data_status_success)
     {
-        rocsparse::log_debug(handle, rocsparse::to_string(*data_status));
+        rocsparse::log_debug(handle, rocsparse::enum_utils::to_string(*data_status));
     }
 
     return rocsparse_status_success;
@@ -282,6 +194,8 @@ namespace rocsparse
                                                            rocsparse_data_status* data_status,
                                                            void*                  temp_buffer)
     {
+        ROCSPARSE_ROUTINE_TRACE;
+
         return rocsparse_status_continue;
     }
 }
@@ -304,6 +218,7 @@ rocsparse_status rocsparse::check_matrix_gebsr_checkarg(rocsparse_handle       h
                                                         rocsparse_data_status* data_status, //14
                                                         void*                  temp_buffer) //15
 {
+    ROCSPARSE_ROUTINE_TRACE;
 
     ROCSPARSE_CHECKARG_HANDLE(0, handle);
     ROCSPARSE_CHECKARG_ENUM(1, dir);
@@ -333,7 +248,7 @@ rocsparse_status rocsparse::check_matrix_gebsr_checkarg(rocsparse_handle       h
         {
             rocsparse::log_debug(handle,
                                  ("Matrix was specified to be "
-                                  + std::string(rocsparse::to_string(matrix_type))
+                                  + std::string(rocsparse::enum_utils::to_string(matrix_type))
                                   + " but (row_block_dim != col_block_dim || mb != nb)"));
         }
     }
@@ -438,6 +353,7 @@ INSTANTIATE(int64_t, int64_t, rocsparse_double_complex);
                                      void*                  temp_buffer)                         \
     try                                                                                          \
     {                                                                                            \
+        ROCSPARSE_ROUTINE_TRACE;                                                                 \
         RETURN_IF_ROCSPARSE_ERROR(                                                               \
             (rocsparse::check_matrix_gebsr_impl<T, rocsparse_int, rocsparse_int>(handle,         \
                                                                                  dir,            \
